/*
  Name: CMPgame.cpp
  Copyright: Polecatsoft 2005
  Game: Mercenary Soldier
  Author: Master Lenman
  Date: -
  Description: This is for the client part of multiplayer games
*/

#include <allegro.h>
#include <string>
#include <fstream>
using namespace std;

#include "ElementaryFunctions.h"
#include "mapClass.h"
#include "CMPgame.h"
#include "Global.h"
#include "keyStatesClass.h"
#include "Errors.h"
#include "packet.h"

clientMultiplayerGame::clientMultiplayerGame(string name)
{
	lifeBar = loadBitmap("gfx/lifebar.tga");
	armorBar = loadBitmap("gfx/armorbar.tga");
	//Let game run in background
	if(set_display_switch_mode(SWITCH_BACKGROUND))
		errors::fatalError("Failed to switch into background mode!");
	serverConn = net_openconn(global.curDriver, NULL);
	if(!serverConn)
	    errors::networkError("Unable to open server connection");
	if(net_connect(serverConn, name.c_str()))
		errors::networkError(string("Can't connect to ") + name);
	connected = false;
	exit = false;
	waitingForConnection = 0;
	messageNeededCountdown = 32;

	int i;
	for(i = 0; i < WEAPONTYPES - 2; ++i)
	{
		//char(48) == '0', so i is added to get the paths to bullet1.tga, bullet2.tga...
		string path = string("gfx/weapon") + char(48 + i) + string(".tga");
		weaponSprites[i] = loadBitmap(path);
	}
	DATAFILE *datfile = load_datafile("data/playergr.dat");
	if(!datfile)
		errors::datafileError("data/playergr.dat");
	sprites = new BITMAP*[MAXSPRITES];
	for(i = 0; i < MAXSPRITES; ++i)
		sprites[i] = createDatafileBitmap(datfile, i);
	unload_datafile(datfile);
	curWeapon = 0;
	players = 0;
	myID = -1;
}

clientMultiplayerGame::~clientMultiplayerGame()
{
	if(destroyMessageClass)
	    delete messages;
	net_closeconn(serverConn);
	destroy_bitmap(lifeBar);
	destroy_bitmap(armorBar);
	int i;
	for(i = 0; i < WEAPONTYPES - 2; ++i)
		destroy_bitmap(weaponSprites[i]);
	if(players)
		delete [] players;
	for(int i = 0; i < MAXSPRITES; ++i)
	{
		destroy_bitmap(sprites[i]);
	}
	delete [] sprites;
}

void clientMultiplayerGame::exeLogic()
{
	int i;
	if(!connected)
	{
		if(checkConnection())
		{
			connected = true;
			pack.reset();
			pack.put(P_REQUEST);
			pack.put(MAJOR_VERSION);
			pack.put(MINOR_VERSION);
			pack.put(VERSION_LETTER);
			for(i = 0; i < 8; ++i)
			    pack.put(0);
			pack.send(serverConn);
			waitingForConnection = 0;
			messages->sendText("Connection established!");
			messages->output();
//			timerCounter = 16;
		}
		else
		    ++waitingForConnection;
	}
	if(connected)
	{
		//Buffer for packet length
		int length;
		//Did at least 1 packet arrive?
		bool once = false;
		while(length = pack.receive(serverConn))
		{
			waitingForConnection = 0;
			once = true;
			char packetType = pack.get();
			switch(packetType)
			{
			case P_POSUPDATE:
			{
				char flag = pack.get();
				for(i = 0; i < maxPlayers; ++i)
				{
					if(!(flag & (1 << i)))
					    continue;
					pack.read(f2ch(players[i].getFlPosX()), sizeof(float));
					pack.read(f2ch(players[i].getFlPosY()), sizeof(float));
					players[i].getState() = pack.get();
					players[i].getDirection() = pack.get();
					players[i].getPhase() = pack.get();
				}
			}
				break;
			case P_NEWPLAYER:
				{
					i = pack.get();
					int points = pack.get();
					char tmp[9];
					pack.read(tmp, 8);
					tmp[8] = 0;
					players[i].join(tmp, points);
					if(i == myID)
					    break;
					if(pack.get())
						messages->sendText("New player:");
					else
					    messages->sendText("Player on server:");
					messages->sendText(tmp);
					messages->output();
				}
				break;
			case P_PLAYERLEFT:
				{
					i = pack.get();
				    messages->sendText("Player left:");
					messages->sendText(players[i].getName());
					messages->output();
					players[i].leave();
				}
				break;
			case P_ACCEPT:
			{
				myID = pack.get();
				maxPlayers = pack.get();
				maxArmor = armor = pack.get();
				maxLife = life = pack.get();
				players = new mpcUnit[maxPlayers];
				for(i = 0; i < maxPlayers; ++i)
					players[i].setSprites(sprites);
				loadMap();
			}
				break;
			case P_SERVERSHUTDOWN:
				exit = true;
				messages->sendText("Server was shut down!");
				messages->output();
				break;
			case P_FULL:
				exit = true;
				messages->sendText("Server is full!");
				messages->output();
				break;
			case P_VERSIONMISMATCH:
				messages->sendText("The server is running a different Mercenary Soldier version!");
				messages->output();
				exit = true;
				break;
			};
		}

		if(!once)
		    ++waitingForConnection;
		if(!(--messageNeededCountdown))
		{
			pack.reset();
			pack.put(P_NOP);
			pack.send(serverConn);
			messageNeededCountdown = 32;
		}
		
		if(myID > -1)
		{
			players[myID].updatePhase();
			//Actual game controls
			bool sendMoveUpdate = false;
			struct mpRequest moveRequest;
			moveRequest.moveRequestFlag = 0;
			//Turn left
			if(key[global.keyLeft])
			{
				players[myID].getDirection() -= 2;
				if(key[global.keyAim])
				{
					//Turn slower
					if(!SPREAD[curWeapon])
					++players[myID].getDirection();
				}
				else if(key[global.keyBoost])
				{
					//Turn faster
					players[myID].getDirection() -= 3;
				}
				players[myID].getState() = STATE_STAND;
				sendMoveUpdate = true;
			}
			//Turn right
			if(key[global.keyRight])
			{
				players[myID].getDirection() += 2;
				if(key[global.keyAim])
				{
				//Turn slower
					if(!SPREAD[curWeapon])
						--players[myID].getDirection();
				}
				else if(key[global.keyBoost])
				{
					//Turn faster
					players[myID].getDirection() += 3;
				}
				players[myID].getState() = STATE_STAND;
				sendMoveUpdate = true;
			}
			//Don't let angle get bigger than 255
			players[myID].getDirection() &= 0xFF;
			if(key[global.keyUp] || key[global.keyDown])
			{
				//float newX = std::sin(deg2rad(players[myID].getDirection())) * 3.0;
				//float newY = std::cos(deg2rad(players[myID].getDirection())) * -3.0;
				moveRequest.moveRequestFlag = 1;
				if(key[global.keyDown])
				{
					//newX *= -0.75;
					//newY *= -0.75;
					moveRequest.moveRequestFlag |= 2;
				}
				//moveRequest.newPosX = players[myID].getFlPosX() + newX;
				//moveRequest.newPosY = players[myID].getFlPosY() + newY;
				players[myID].getState() = STATE_WALK;
				sendMoveUpdate = true;
			}
			if(!sendMoveUpdate && players[myID].getState() != STATE_STAND)
			{
				players[myID].getState() = STATE_STAND;
				sendMoveUpdate = true;
			}
			if(sendMoveUpdate)
			{
				moveRequest.phase = players[myID].getPhase();
				moveRequest.dir = players[myID].getDirection();
				moveRequest.curSprite = players[myID].getState();
				pack.reset();
				pack.put(P_MOVEREQ);
				pack.write(reinterpret_cast<char*>(&moveRequest), sizeof(mpRequest));
				pack.send(serverConn);
			}
		}
	}
	messages->poll();
}

void clientMultiplayerGame::exeGraphic()
{
	if(!connected)
	{
		clear_to_color(global.doublebuffer, makecol(0xFF, 0xFF, 0xFF));
		messages->draw(30, true);
	}
	else
	{
		int x = players[myID].getPosX(), y = players[myID].getPosY();
		map.draw(x, y);
		for(int i = 0; i < maxPlayers; ++i)
		    players[i].draw(x, y, i);
		//Ammo and weapon info
		//textprintf_centre_ex(global.doublebuffer, global.size20, (weaponSprites[curWeapon]->w >> 1) + 5, 3, 0, -1, "%i", player.getAmmoleft());
		masked_blit(weaponSprites[curWeapon], global.doublebuffer, 0, 0, 5, text_height(global.size20) + 6, weaponSprites[curWeapon]->w, weaponSprites[curWeapon]->h);
		//messages
		messages->draw(text_height(global.size20) + 10 + weaponSprites[curWeapon]->h, key[global.keyMessages]);
		//Life and armor info
		blit(lifeBar, global.doublebuffer, 0, 0, SCREEN_W - 8 - getLifePercentage() * 2, 8, getLifePercentage() * 2, lifeBar->h);
		rect(global.doublebuffer, SCREEN_W - 208, 8, SCREEN_W - 8, 8 + lifeBar->h, 0);
		blit(armorBar, global.doublebuffer, 0, 0, SCREEN_W - 8 - getArmorPercentage() * 2, 16 + lifeBar->h, getArmorPercentage() * 2, armorBar->h);
		rect(global.doublebuffer, SCREEN_W - 208, 16 + lifeBar->h, SCREEN_W - 8, 16 + lifeBar->h + armorBar->h, 0);
	}
}

int clientMultiplayerGame::getWFC()
{
	return waitingForConnection;
}

bool clientMultiplayerGame::wantsExit()
{
	return exit;
}

int clientMultiplayerGame::getLifePercentage()
{
	return int(float(life) / (float(maxLife) / 100.0));
}

int clientMultiplayerGame::getArmorPercentage()
{
	return int(float(armor) / (float(maxArmor) / 100.0));
}

//Returns true if there was an answer from the server
bool clientMultiplayerGame::checkConnection()
{
	int x = net_poll_connect(serverConn);
	if(x > 0)
		return true;
	if(x < 0)
	    errors::networkError("Connection could not be established!");
	return false;
}

int clientMultiplayerGame::operator()()
{
	bool run = true;
	int curSkip = 0;
	int quitWaitTime = 0;
	int quitCountdown = 0;
	setMessageClass(0);
	messages->sendText("Beginning to connect...");
	messages->output();
	//allegro_message("%i", timerCounter);
	//timerCounter = 8;
	int graphCountdown = 8;
	while(run)
	{
		while(timerCounter > 0)
		{
			if(!quitCountdown)
				exeLogic();
			if(!connected && waitingForConnection > 96 && !quitCountdown)
			{
				messages->sendText("Connection attempt timed out!");
				messages->output();
			    timerCounter = 0;
			    quitCountdown = 64;
			}
			else if(connected && waitingForConnection > 64 && !quitCountdown)
			{
				messages->sendText("Lost connection to server!");
				messages->output();
			    timerCounter = 0;
			    quitCountdown = 64;
			}
			else if(exit && !quitCountdown)
			{
				connected = false;
			    quitCountdown = 128;
			}
			if(keyStates[KEY_ESC])
			{
				if(quitWaitTime)
				    run = false;
				else
				    quitWaitTime = 96;
			}
			else if(quitWaitTime)
			    --quitWaitTime;

			if(quitCountdown)
			{
				if(!(--quitCountdown))
				    run = false;
			}

			if(graphCountdown)
			    --graphCountdown;

			--timerCounter;
			++curSkip;
			if(curSkip >= global.frameskip)
			{
				timerCounter = 0;
				curSkip = 0;
			}
		}
		curSkip = 0;
		if(graphCountdown)
		    continue;
		clear(global.doublebuffer);
		exeGraphic();
		showStatistics();
		if(quitWaitTime)
		    textout_centre_ex(global.doublebuffer, global.size20, "Press esc again to quit.", SCREEN_W >> 1, 20, makecol(0xFF, 0xFF, 0xFF), 0);
		if(global.vsync)
			vsync();
		show();
	}
	sendExitPacket();
	return 0;
}

void clientMultiplayerGame::loadMap()
{
	string mapname;
	while(pack.peak() != 0)
	    mapname.append(1, pack.get());
	string mapPath = string("mpmaps/") + mapname + string(".msm");
	ifstream readfile(mapPath.c_str(), ios::binary);
	if(readfile.fail())
	{
		sendExitPacket();
		errors::mapLoadError(mapPath, "File not found");
	}
	gameType = map.loadMap(readfile, mapname);
}

void clientMultiplayerGame::setMessageClass(messageClass *msc)
{
	destroyMessageClass = msc != 0;
	if(!msc)
	    messages = new messageClass(32, 7, 3, global.size12, global.size12bold, global.doublebuffer);
	else
	    messages = msc;
}

void clientMultiplayerGame::sendExitPacket()
{
	pack.reset();
	pack.put(P_SERVERSHUTDOWN);
	pack.send(serverConn);
}

void clientMultiplayerGame::showStatistics()
{
	if(key[global.keyMission])
	{
		int active = 0;
		int i;
		for(i = 0; i < maxPlayers; ++i)
		{
			if(!players[i].isActive())
			    continue;
			++active;
		}
		rectfill(global.doublebuffer, 60, 60, 740, (active + 1) * 20 + 80, 0);
		textout_centre_ex(global.doublebuffer, global.size20, (
		string("Connected to ") + net_getpeer(serverConn) + string(", ") + i2str(active) + " of " + i2str(maxPlayers) + " players"
		).c_str(), SCREEN_W >> 1, 60, makecol(0xFF, 0xFF, 0xFF), -1);
		active = 0;
		for(i = 0; i < maxPlayers; ++i)
		{
			if(!players[i].isActive())
			    continue;
			FONT *useFont = global.size12;
			if(myID == i)
			    useFont = global.size12bold;
			textout_centre_ex(global.doublebuffer, useFont, (
			players[i].getName() + " - " + i2str(players[i].getPoints()) + " points"
			).c_str(), SCREEN_W >> 1, active * 19 + 90, makecol(0xFF, 0xFF, 0xFF), 0);
			++active;
		}
	}
}
