/*
  Name: SPgame.cpp
  Copyright: Polecatsoft 2005
  Game: Mercenary Soldier
  Author: Master Lenman
  Date: -
  Description: The game class is the most important class,
    It contains all other classes needed in game and
	makes communication between different classes possible
*/

#include <allegro.h>
#include <almp3.h>
#include <libnet.h>
#include <string>
#include <fstream>
#include <vector>
#include <cmath>
using namespace std;

#include "Sound.h"
#include "messageClass.h"
#include "ElementaryFunctions.h"
#include "requests.h"
#include "Global.h"
#include "bytecode.h"
#include "Entity.h"
#include "Unit.h"
#include "playerUnit.h"
#include "Enemy.h"
#include "Civilian.h"
#include "Compiler.h"
#include "VM.h"
#include "mapClass.h"
#include "bullet.h"
#include "SPgame.h"
#include "Errors.h"
#include "keyStatesClass.h"
#include "Entity.h"
#include "Unit.h"
#include "menues.h"
#include "mouse.h"

//Maximum number of bullets that can be created
const int MAX_BULLETS = 1024;

//string weapons[6] = { "Pistol", "Shotgun", "MG", "Sniper", "Grenades", "Rocket Launcher" };

const int TYPE_PLAYER = -1,
TYPE_UNIT = 0,
TYPE_BULLET = -2;

int singlePlayerGame::operator()(string &returnValue)
{
	//Hold the position the map is drawn on
	int x = 0, y = 0;
	int i;
	int retNum;
	int curSkip = 0;
	int &curObjective = VM.getVar(33) = 1;
	bool running = true;
	bool needsRefresh = true;
	int nextMapCountdown = -1;
	BITMAP *gameOver = loadBitmap("gfx/gameover.tga"),
		*missionSolved = loadBitmap("gfx/missionsolved.tga");
	BITMAP *endGameScreen = NULL;
	while(running)
	{
////////////////////////////////////////////////////////////
/////////////////////// LOGIC //////////////////////////////
////////////////////////////////////////////////////////////
		rest(3);
		while(timerCounter > 0)
		{
			moveRequest *mReq;
			bulletRequest *bReq;
			if(player.isAlife())
			{
				mReq = player.getMoveRequest(x, y);
				map.handleMoveRequest(mReq, VM, keyStates[global.keyUse]);
				this->handleMoveRequest(mReq, VM, TYPE_PLAYER);

				bReq = player.getBulletRequest();
				addBullets(bReq);

				player.takeRequests(mReq, bReq);
			}

			for(i = 0; i < MAX_BULLETS; ++i)
			{
				//mReq is reused
				if(!bullets[i].isActive())
					continue;
				mReq = bullets[i].update();
				map.handleMoveRequest(mReq, VM, 0);
				this->handleMoveRequest(mReq, VM, TYPE_BULLET - bullets[i].getType(), bullets[i].getTeam(), bullets[i].getDirection());
				if(!bullets[i].returnRequest(mReq))
				{
					bulletRequest bReq;
					bReq.set(mReq->getNewPosX(), mReq->getNewPosY(), 0, bullets[i].getType() + 2, 0);
					delete [] mReq;

					explosion(bReq);
				}
			}
			//if no unit will say that it's still alife, we know that all are dead
			VM.getVar(0) = true;
			//Number of enemies still alife
			VM.getVar(13) = 0;
			//Get move Requests from all Units
			for(i = 0; i < nUnit; ++i)
			{
				if(units[i]->isAlife())
				{
					//Check if all enemies are dead
					if(units[i]->isEnemy())
					{
						VM.getVar(0) = false;
						++VM.getVar(13);
					}

					if(player.isAlife())
						mReq = units[i]->getMoveRequest(player.getPosX(), player.getPosY());
					else
					    mReq = units[i]->getMoveRequest(-1, -1);

					map.handleMoveRequest(mReq, VM, 0);
					this->handleMoveRequest(mReq, VM, TYPE_UNIT + i);

					if(player.isAlife())
						addBullets(bReq = units[i]->getBulletRequest());
					else
					    bReq = NULL;
					units[i]->takeRequests(mReq, bReq);
				}
			}

			if(globalScript)
				VM.compute(*globalScript);

			if(nextMapCountdown > 0)
				--nextMapCountdown;
			else if(!nextMapCountdown)
				running = false;
			//Should we load the next map?
			else if(VM.needNextMap())
			{
				returnValue = VM.getMapString();
				retNum = 1;
				nextMapCountdown = 128;
				if(returnValue == string("GO"))
					endGameScreen = gameOver;
				else
					endGameScreen = missionSolved;
			}
			//Update messages
			messages->poll();
			//Update map animation
			map.poll();
			//Update mp3 files
			VM.pollSounds();

			if(keyStates[global.keyMission])
				reviewMissionDescription();

			if(keyStates[global.keyMessages])
				messages->showArchive();

			//Mission objectives
			if(keyStates[KEY_1])
				curObjective = 1;
			if(keyStates[KEY_2])
				curObjective = 2;
			if(keyStates[KEY_3])
				curObjective = 3;
			if(keyStates[KEY_4])
				curObjective = 4;
			if(keyStates[KEY_5])
				curObjective = 5;

			if(curObjective > objectives)
				curObjective = objectives;
			if(curObjective <= 0)
				curObjective = 1;

			if(keyStates[KEY_ESC])
			{
				bool menuRun = true;
				BITMAP *background = createBitmap(SCREEN_W, SCREEN_H);
				blit(global.doublebuffer, background, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
				while(menuRun)
				{
					int switchVal = menu::inGameMenu(background);
					switch(switchVal)
					{
					case 0:
						menuRun = false;
						break;
					case 1:
						reviewMissionDescription();
						break;
					case 2:
						menu::options(false);
						VM.updateSoundVol();
						break;
					case 3:
						retNum = 0;
						running = false;
						menuRun = false;
						break;
					}
				}
				destroy_bitmap(background);
			}
			//Cheats
			if(keyStates[KEY_F10])
			{
				global.showBlood ^= 1;
				if(global.showBlood)
					messages->sendText("Blood cheat activated");
				else
					messages->sendText("Blood cheat deactivated");

				messages->output();
			}
			if(keyStates[KEY_F11])
			{
				for(i = 0; i < 5; ++i)
					VM.getVar(i + 23) = 999;
				messages->sendText("Ammo cheat used");
				messages->output();
			}
			if(keyStates[KEY_F12])
			{
				godCheat ^= 1;
				if(godCheat)
					messages->sendText("God mode activated");
				else
					messages->sendText("God mode deactivated");

				messages->output();
			}
			if(nextMapCountdown > 0)
			    godCheat = 1;

			needsRefresh = true;

			--timerCounter;
			++curSkip;
			if(curSkip >= global.frameskip)
				timerCounter = 0;
		}
////////////////////////////////////////////////////////////
////////////////////// GRAPHIC /////////////////////////////
////////////////////////////////////////////////////////////
		if(needsRefresh)
		{
			clear(global.doublebuffer);
			map.draw(x = player.getPosX(), y = player.getPosY());
			for(i = 0; i < nUnit; ++i)
			{
				units[i]->draw(x, y);
			}
			for(i = 0; i < MAX_BULLETS; ++i)
				bullets[i].draw(x, y);
			player.draw(x, y);

			//Ammo and weapon info
			textprintf_centre_ex(global.doublebuffer, global.size20, (weaponSprites[player.getCurWeapon()]->w >> 1) + 5, 3, 0, -1, "%i", player.getAmmoleft());
			masked_blit(weaponSprites[player.getCurWeapon()], global.doublebuffer, 0, 0, 5, text_height(global.size20) + 6, weaponSprites[player.getCurWeapon()]->w, weaponSprites[player.getCurWeapon()]->h);

			//messages
			messages->draw(text_height(global.size20) + 10 + weaponSprites[player.getCurWeapon()]->h);

			//Life and armor info
			blit(lifeBar, global.doublebuffer, 0, 0, SCREEN_W - 8 - player.getLifePercentage() * 2, 8, player.getLifePercentage() * 2, lifeBar->h);
			rect(global.doublebuffer, SCREEN_W - 208, 8, SCREEN_W - 8, 8 + lifeBar->h, 0);
			if(player.getMaxarmor())
			{
				blit(armorBar, global.doublebuffer, 0, 0, SCREEN_W - 8 - player.getArmorPercentage() * 2, 16 + lifeBar->h, player.getArmorPercentage() * 2, armorBar->h);
				rect(global.doublebuffer, SCREEN_W - 208, 16 + lifeBar->h, SCREEN_W - 8, 16 + lifeBar->h + armorBar->h, 0);
			}

			//Mission direction arrow
			rotate_sprite(global.doublebuffer, dirArrow, SCREEN_W - dirArrow->w - 3, SCREEN_H - dirArrow->h - 3, fixatan2(itofix(objectivePosYs[curObjective - 1] - player.getPosY()), itofix(objectivePosXs[curObjective - 1] - player.getPosX())) & 0xFFFFFF);
			textprintf_centre_ex(global.doublebuffer, global.size12, SCREEN_W - ((dirArrow->w + text_length(global.size12, "0")) >> 1) - 3, SCREEN_H - ((dirArrow->h + text_height(global.size12)) >> 1) - 3, makecol(0xFF, 0xFF, 0xFF), -1, "%i", curObjective);
			//rotate_sprite(global.doublebuffer, dirArrow, SCREEN_W - dirArrow->w - 3, SCREEN_H - dirArrow->h - 3, (fixatan2(itofix(map.getMapHeight()), itofix(map.getMapWidth())) + itofix(64)) & 0xFFFFFF);
			//allegro_message("playerX: %i, targetX: %i, playerY: %i, targetY: %i, angle: %i", player.getPosX(), objectivePosXs[0], player.getPosY(), objectivePosYs[curObjective], fixtoi(fixatan2(itofix(objectivePosYs[curObjective] - player.getPosY()), itofix(objectivePosXs[0] - player.getPosX()))));
			
			if(endGameScreen)
				masked_blit(endGameScreen, global.doublebuffer, 0, 0, (SCREEN_W - endGameScreen->w) >> 1, (SCREEN_H - endGameScreen->h) >> 1, endGameScreen->w, endGameScreen->h);
			else
				masked_blit(global.crosshair, global.doublebuffer, 0, 0, mouse_x - 6, mouse_y - 6, global.crosshair->w, global.crosshair->h);
			if(global.vsync)
				vsync();
			show();
			curSkip = 0;

			needsRefresh = false;
		}
	}
	destroy_bitmap(missionSolved);
	destroy_bitmap(gameOver);
	return retNum;
}

void singlePlayerGame::handleMoveRequest(moveRequest *mReq, VirtualMachine &VM, int type, void *bulletTeam, int bulletDirection)
{
	if(!mReq)
		return;
	if(mReq->wasDeclined())
		return;
	if(type != TYPE_PLAYER)// && !(type < TYPE_PLAYER && bullets[(type - TYPE_BULLET) * -1].getTeam() == 0 && bullets[(type - TYPE_BULLET) * -1].isImmune()))
	{
		if(collides(int(mReq->getNewPosX()) - mReq->getRadius(), int(mReq->getNewPosY()) - mReq->getRadius(), int(mReq->getNewPosX()) + mReq->getRadius(), int(mReq->getNewPosY()) + mReq->getRadius(),
			player.getPosX() - player.getRadius(), player.getPosY() - player.getRadius(), player.getPosX() + player.getRadius(), player.getPosY() + player.getRadius()))
		//if(player.getRadius() + mReq->getRadius() > int(sqrt((player.getFlPosX() - mReq->getNewPosX()) * (player.getFlPosX() - mReq->getNewPosX())
		//+ (player.getFlPosY() - mReq->getNewPosX()) * (player.getFlPosY() - mReq->getNewPosY()))))
		{
			//Decline if the requester is a unit
			if(type > TYPE_PLAYER)
				mReq->decline();
			//Decline and get hit if the requester is a bullet
			else if(bulletTeam != (void*)&player)
			{
				mReq->decline();
				if(!godCheat)
					player.getHit((type - TYPE_BULLET) * -1);
			}
			if(!player.isAlife())
				VM.compute(dieCode);
			return;
		}
	}
	for(int i = 0; i < nUnit; ++i)
	{
		if(type == i || !units[i]->isAlife())
			continue;
		if(collides(int(mReq->getNewPosX()) - mReq->getRadius(), int(mReq->getNewPosY()) - mReq->getRadius(), int(mReq->getNewPosX()) + mReq->getRadius(), int(mReq->getNewPosY()) + mReq->getRadius(),
		units[i]->getPosX() - units[i]->getRadius(), units[i]->getPosY() - units[i]->getRadius(), units[i]->getPosX() + units[i]->getRadius(), units[i]->getPosY() + units[i]->getRadius()))
		//if(units[i]->getRadius() + mReq->getRadius() > int(sqrt((units[i]->getFlPosX() - mReq->getNewPosX()) * (units[i]->getFlPosX() - mReq->getNewPosX())
		//+ (units[i]->getFlPosY() - mReq->getNewPosX()) * (units[i]->getFlPosY() - mReq->getNewPosY()))))
		{
			if(type >= TYPE_PLAYER)
				mReq->decline();
			else if(bulletTeam != (void*)units[i]) //Team 1 are all enemies and also civilians
			{
				mReq->decline();
				if(bulletTeam == (void*)&player || !units[i]->isEnemy() || !bulletTeam)
					units[i]->getHit((type - TYPE_BULLET) * -1);
			}
			if(!units[i]->isAlife())
			{
				units[i]->getDirection() = (bulletDirection + 128) & 0xFF;
				if(onKill[i])
					VM.compute(*onKill[i]);
			}
			return;
		}
	}
}

singlePlayerGame::~singlePlayerGame()
{
	int i;
	for(i = 0; i < nUnit; ++i)
		delete units[i];
	delete [] units;

	delete [] bullets;
	if(enemyBasetype)
		delete [] enemyBasetype;
	if(civilianBasetype)
		delete [] civilianBasetype;
	if(onKill)
	{
		for(i = 0; i < nUnit; ++i)
		{
			if(!origCode[i])
				continue;
			if(onKill[i])
				delete onKill[i];
		}
		delete [] onKill;
	}
	if(onCall)
	{
		for(i = 0; i < nUnit; ++i)
		{
			if(!origCode[i])
				continue;
			if(onCall[i])
				delete onCall[i];
		}
		delete [] onCall;
	}
	delete [] origCode;

	if(globalScript)
		delete globalScript;

	if(objectivePosXs)
		delete [] objectivePosXs;
	if(objectivePosYs)
		delete [] objectivePosYs;

	destroy_bitmap(lifeBar);
	destroy_bitmap(armorBar);
	destroy_bitmap(dirArrow);
	for(i = 0; i < WEAPONTYPES - 2; ++i)
		destroy_bitmap(weaponSprites[i]);

	destroy_bitmap(missionDesc);

	delete messages;
}

singlePlayerGame::singlePlayerGame(string name, int healRefills, int ammoRefills) : maxSizeX(270), maxSizeY(230)
{
	//Show a message telling the user that the map is beeing loaded
	clear(global.doublebuffer);
	textout_centre_ex(global.doublebuffer, global.size20, "Loading map data, please wait...", SCREEN_W >> 1, SCREEN_H >> 1, makecol(0xFF, 0xFF, 0xFF), -1);
	show();

	//Copy values for saving issues
	healRefillPlus = healRefills;
	ammoRefillPlus = ammoRefills;

	//Create message class
	messages = new messageClass(32, 7, 3, global.size12, global.size12bold, global.doublebuffer);
	VM.setMessageClass(messages);

	bullets = new bullet[MAX_BULLETS];

	string mapPath = "maps/" + name + ".msm";
	ifstream readfile(mapPath.c_str(), ios::binary);
	//ifstream readfile("maps/map1.msm", ios::binary);
	if(readfile.fail())
		errors::mapLoadError(mapPath, "Failed to open file");
	map.loadMap(readfile, mapPath, 'S', VM);

	//Load the global script
	char cBuffer;
	readfile.read(&cBuffer, 1);
	bool preComp = false;
	ifstream *globScriptFile;
	const int BUFFERSIZE = 0x800;
	unsigned char *fullCode = NULL;
	int outputSize = BUFFERSIZE;
	if(cBuffer)
	{
		globScriptFile = &readfile;
		fullCode = new unsigned char[BUFFERSIZE];
		preComp = true;
		cBuffer = 0;
	}
	if(!cBuffer)
	{
		string globScriptPath = string("scripts/") + name + string("global.ums");
		if(!fullCode && fileExists(globScriptPath))
		{
			globScriptFile = NULL;
			fullCode = new unsigned char[BUFFERSIZE];
			if(!compile(globScriptPath, "", fullCode, &outputSize))
				errors::scriptError("An error occured in compile stage.\nSee compiler.log for details.");
		}
		else
		{
			globScriptPath = string("scripts/") + name + string("global.mss");
			if(fullCode || fileExists(globScriptPath))
			{
				if(!fullCode)
				{
					fullCode = new unsigned char[BUFFERSIZE];
					globScriptFile = new ifstream(globScriptPath.c_str(), ios::binary);
				}
				while(true)
				{
					globScriptFile->read((char *)(&fullCode[outputSize]), 1);
					if(fullCode[outputSize] == SCRIPTEND)
						break;
					++outputSize;
					if(outputSize >= BUFFERSIZE)
					{
						errors::scriptError("Script too big");
					}
				}
			}
			else
				globScriptFile = NULL;
		}
	}

	if(fullCode)
	{
		int i = 0;
		if(!preComp && globScriptFile)
		{
			globScriptFile->close();
			delete globScriptFile;
		}
		bytecode codeBuffer;
		codeBuffer.extract(fullCode, GLOBAL);
		delete [] fullCode;
		if(codeBuffer.getLength())
		{
			globalScript = new bytecode;
			*globalScript = codeBuffer;
		}
		else
			globalScript = NULL;
	}
	else
		globalScript = NULL;

	//Campaign information
	readfile.read(&cBuffer, 1);
	if(cBuffer)
	{
		string temp;
		getline(readfile, temp, DELIMITER);
		showMissionDescription(temp, true);
	}
	string charsetPath;
	getline(readfile, charsetPath, DELIMITER);
	ifstream charset(charsetPath.c_str());
	if(charset.fail())
		errors::mapLoadError(mapPath, "Failed to open set of characters");
	int i;
	int playerTypes;
	charset >> playerTypes;
	if(playerTypes > 1)
		errors::mapLoadError(mapPath, "Character set contains multiple player definitions");
	for(i = 0; i < playerTypes; ++i)
	{
		//The charset Path is being misused as a temporary Buffer for the character's name
		charset >> charsetPath;

		float speed;
		charset >> speed;
		int maxLife = 0, maxArmor = 0, armorType = 0;
		charset >> maxLife >> maxArmor;
		if(maxArmor)
			charset >> armorType;
		//Set globals for the Virtual Machine
		VM.getVar(6) = maxLife;
		VM.getVar(7) = maxArmor;

		//Read ammo Info in Virtual Machine variables and send pointers to the player
		for(int j = 0; j < 5; ++j)
		{
			int muniValue;
			charset >> muniValue;
			if(VM.getVar(j + 23) == 0)
				VM.getVar(j + 23) = muniValue;
			else if(VM.getVar(j + 23) == -1)
			    VM.getVar(23 + j) = 0;
		}
		string spritePath;
		charset >> spritePath;
		player.addressInit(&VM.getVar(23), &VM.getVar(28), &VM.getVar(29), &VM.getVar(34));
		player.load(speed, maxLife, maxArmor, armorType, spritePath);
	}

	charset >> enemyTypes;
	if(enemyTypes)
		enemyBasetype = new Enemy[enemyTypes];
	else
		enemyBasetype = NULL;
	for(i = 0; i < enemyTypes; ++i)
	{
		charset >> charsetPath;

		float speed;
		charset >> speed;
		int maxLife, maxArmor, armorType = 0;
		charset >> maxLife >> maxArmor;
		if(maxArmor)
			charset >> armorType;
		int weapon;
		charset >> weapon;
		int shPause;
		charset >> shPause;
		if(ammoRefillPlus && shPause)
		{
			shPause -= ammoRefillPlus >> 1;
			if(ammoRefillPlus & 1)
				shPause -= 1;
			if(shPause <= 0)
				shPause = 1;
		}
		int tD;
		charset >> tD;
		string spritePath;
		charset >> spritePath;
		enemyBasetype[i].load(speed, maxLife, maxArmor, armorType, spritePath, tD, weapon, shPause);
	}

	charset >> civilianTypes;
	if(civilianTypes)
		civilianBasetype = new Civilian[civilianTypes];
	else
		civilianBasetype = NULL;
	for(i = 0; i < civilianTypes; ++i)
	{
		charset >> charsetPath;

		float speed;
		charset >> speed;
		int maxLife, maxArmor, armorType = 0;
		charset >> maxLife >> maxArmor;
		if(maxArmor)
			charset >> armorType;
		string spritePath;
		charset >> spritePath;

		civilianBasetype[i].load(speed, maxLife, maxArmor, armorType, spritePath, 0, 0);
	}
	charset.close();

	readfile.read(i2ch(nUnit), sizeof(int));
	ifstream *scriptfile = &readfile;
	if(nUnit < 1)
		errors::mapLoadError(mapPath, "No Units on map!");
	units = new Unit*[nUnit - 1];
	onCall = new bytecode*[nUnit - 1];
	onKill = new bytecode*[nUnit - 1];
	bytecode **onLoad = new bytecode*[nUnit - 1];
	origCode = new bool[nUnit - 1];
	int pos = 0;

	string lastFive[5];
	for(i = 0; i < nUnit; ++i)
	{
		char type;
		float initPosX, initPosY;
		char initDir, initState;
		char script;
		readfile.read(&type, 1);
		readfile.read(f2ch(initPosX), sizeof(float));
		readfile.read(f2ch(initPosY), sizeof(float));
		readfile.read(&initDir, 1);
		readfile.read(&initState, 1);
		readfile.read(&script, 1);

		//The player
		if(!type)
		{
			player.initialise(initPosX, initPosY, initDir, initState);

			//Standard onKill code for player
			unsigned char dieBytecode[8] = { 0x01, 0x05, 0x0E, 0x47, 0x4F, 0x00, 0xFE, 0xFF };
			dieCode.extract(dieBytecode, ONKILL);

			if(script)
				errors::mapLoadError(mapPath, "Scripts for the player character are illegal!");

			continue;
		}
		else
			--type;

		//Is it an enemy type
		if(type < enemyTypes)
		{
			units[pos] = new Enemy;
			units[pos]->basetypeGet(enemyBasetype[type]);
			units[pos]->initialise(initPosX, initPosY, initDir, initState);
		}
		else
		{
			units[pos] = new Civilian;
			//units[pos]->setCivilian();
			units[pos]->basetypeGet(civilianBasetype[type - enemyTypes]);
			//units[pos]->basetypeGet(civilianBasetype[0]);
			units[pos]->initialise(initPosX, initPosY, initDir, initState);
		}

		const int BUFFERSIZE = 0x400;
		unsigned char *fullCode; //1KB buffer for script data
		int outputSize = BUFFERSIZE;
		//Load script information
		if(script == 2)
		{
			origCode[pos] = true;
			string scriptPath;
			getline(readfile, scriptPath, DELIMITER);

			//Check if the script was used before
			int j;
			bool found = false;
			for(j = 0; j < 5; ++j)
			{
				//If this is the case, get the script from the other unit that also has the script
				if(lastFive[j] == scriptPath)
				{
					onCall[pos] = onCall[pos - j - 1];
					onKill[pos] = onKill[pos - j - 1];
					onLoad[pos] = onLoad[pos - j - 1];
					if(onLoad[pos])
						VM.compute(*onLoad[pos]);
					origCode[pos] = false;

					++pos;
					found = true;
					break;
				}
			}
			if(found)
				continue;
			for(j = 4; j > 0; --j)
				lastFive[j] = lastFive[j - 1];
			lastFive[0] = scriptPath;

			fullCode = new unsigned char[BUFFERSIZE];
			if(!compile(scriptPath, "", fullCode, &outputSize))
				errors::scriptError("An error occured in compile stage.\nSee compiler.log for details.");
		}
		if(script == 1)
		{
			int position = 0;
			fullCode = new unsigned char[BUFFERSIZE];
			origCode[pos] = true;
			while(true)
			{
				readfile.read((char *)(&fullCode[position]), 1);
				if(fullCode[position] == SCRIPTEND)
					break;
				++position;
				if(position >= BUFFERSIZE)
					errors::scriptError("Script too big");
			}
		}
		if(script)
		{
			//Buffer for code
			bytecode codeBuffer;
			//onLoad
			codeBuffer.extract(fullCode, ONLOAD);
			if(codeBuffer.getLength())
			{
				onLoad[pos] = new bytecode;
				VM.compute(codeBuffer);
				*onLoad[pos] = codeBuffer;
			}
			else
				onLoad[pos] = NULL;
			//onKill
			codeBuffer.extract(fullCode, ONKILL);
			if(codeBuffer.getLength())
			{
				onKill[pos] = new bytecode;
				//Copy content of codeBuffer
				*onKill[pos] = codeBuffer;
			}
			else
				onKill[pos] = NULL;
			//onCall
			codeBuffer.extract(fullCode, ONCALL);
			if(codeBuffer.getLength())
			{
				onCall[pos] = new bytecode;
				*onCall[pos] = codeBuffer;
			}
			else
				onCall[pos] = NULL;
			delete [] fullCode;
		}
		else
		{
			onCall[pos] = NULL;
			onKill[pos] = NULL;
			onLoad[pos] = NULL;
			origCode[pos] = true;
		}

		++pos;
	}
	//Substract 1 from nUnit because the player is also counted
	//and we need a clean nUnit later
	readfile.close();
	--nUnit;
	if(onLoad)
	{
		for(i = 0; i < nUnit; ++i)
		{
			if(!origCode[i])
				continue;
			if(onLoad[i])
				delete onLoad[i];
		}
		delete [] onLoad;
	}

	lifeBar = loadBitmap("gfx/lifebar.tga");
	armorBar = loadBitmap("gfx/armorbar.tga");
	dirArrow = loadBitmap("gfx/arrow.tga");

	//Show a description of the mission
	showMissionDescription(name);

	for(i = 0; i < WEAPONTYPES - 2; ++i)
	{
		//char(48) == '0', so i is added to get the paths to bullet1.tga, bullet2.tga...
		string path = string("gfx/weapon") + char(48 + i) + string(".tga");
		weaponSprites[i] = loadBitmap(path);
	}

	VM.getVar(30) += healRefillPlus;
	VM.getVar(31) += ammoRefillPlus;

	godCheat = false;
}

void singlePlayerGame::showMissionDescription(string name, bool campaign)
{
	objectives = 0;
	string filename = "maps/" + name + ".msd";
	ifstream mission(filename.c_str());
	if(mission.fail())
		errors::mapLoadError(filename, "Failed to open mission description!");

	string readBuffer;
	BITMAP *bg, *buttonSave = loadBitmap("gfx/buttonSave.tga"), *buttonContinue = loadBitmap("gfx/buttonContinue.tga");
	int points[2][4] =
	{
		{ SCREEN_W - buttonSave->w, SCREEN_H - buttonSave->h - buttonContinue->h, SCREEN_W, SCREEN_H - buttonContinue->h },
		{ SCREEN_W - buttonContinue->w, SCREEN_H - buttonContinue->h, SCREEN_W, SCREEN_H }
	};
	missionDesc = createBitmap(SCREEN_W, SCREEN_H);
	bg = loadBitmap("gfx/missionscreen.tga");
	blit(bg, missionDesc, 0, 0, 0, 0, bg->w, bg->h);
	destroy_bitmap(bg);
	if(!campaign)
	{
		bg = map.getMapPreview(maxSizeX, maxSizeY);
		blit(bg, missionDesc, 0, 0, SCREEN_W - maxSizeX + ((maxSizeX - bg->w) >> 1), (maxSizeY - bg->h) >> 1, bg->w, bg->h);
	}
	else
	{
		filename = "gfx/" + name + ".tga";
		bg = loadBitmap(filename);
		blit(bg, missionDesc, 0, 0, SCREEN_W - maxSizeX + ((maxSizeX - bg->w) >> 1), (maxSizeY - bg->h) >> 1, bg->w, bg->h);
		destroy_bitmap(bg);
	}
	int posY = 250;
	if(!campaign)
	{
		textout_centre_ex(missionDesc, global.size20, "Objectives", 660, posY, 0, -1);
		posY += text_height(global.size20) + 6;
		objectiveHeight = posY;
		while(true)
		{
			getline(mission, readBuffer);
			if(readBuffer[0] != '-')
				break;
			++objectives;
			textout_ex(missionDesc, global.size12, readBuffer.c_str(), 530, posY, 0, -1);
			posY += text_height(global.size12) + 3;
		}
		VM.setObjectives(objectives);
		objectivePosXs = new int[objectives];
		objectivePosYs = new int[objectives];
		for(int i = 0; i < objectives; ++i)
		{
			mission >> objectivePosXs[i] >> objectivePosYs[i];
			objectivePosXs[i] *= map.getTilesize();
			objectivePosYs[i] *= map.getTilesize();
		}
		mission.ignore(8, '\n');
	}
	else
		getline(mission, readBuffer);

	posY = 35;
	textout_centre_ex(missionDesc, global.size20, readBuffer.c_str(), 260, posY, 0, -1);
	posY += text_height(global.size20) + 6;
	while(!mission.eof())
	{
		getline(mission, readBuffer);
		textout_justify_ex(missionDesc, global.size12, readBuffer.c_str(), 10, 510, posY, 150, 0, -1);
		posY += text_height(global.size12) + 3;
	}
	if(!campaign)
	{
		//Create default savegame
		ofstream savegame(getSavegamePath(0).c_str());
		savegame << name << endl << healRefillPlus << " " << ammoRefillPlus;
		savegame.close();
	}
	while(true)
	{
		if(timerCounter > 0)
		{
			bool wasClicked = clickEvent();
			if(!campaign && (keyStates[KEY_S] || (wasClicked && mouse_x > points[0][0] && mouse_y > points[0][1] && mouse_y < points[0][3])))
			{
				int slot = menu::savegameSelect(false);
				if(slot >= 0)
				{
					ofstream savegame(getSavegamePath(slot).c_str());
					savegame << name << endl << healRefillPlus << " " << ammoRefillPlus;
					savegame.close();
				}
			}
			if(keyStates[KEY_ENTER] || keyStates[global.keyUse] || (wasClicked && mouse_x >= points[1][0] && mouse_y >= points[1][1]))
				break;
			--timerCounter;
		}
		blit(missionDesc, global.doublebuffer, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
		//if(saved)
		//	textout_centre_ex(global.doublebuffer, global.size20, "Game was saved. Press Enter to continue.", 255, SCREEN_H - 2 - text_height(global.size20), 0, -1);
		//else if(!campaign)
		//	textout_centre_ex(global.doublebuffer, global.size20, "Press S to save or Enter to continue.", 255, SCREEN_H - 2 - text_height(global.size20), 0, -1);
		//else
		//	textout_centre_ex(global.doublebuffer, global.size20, "Press Enter to continue.", 255, SCREEN_H - 2 - text_height(global.size20), 0, -1);
		if(!campaign)
			blit(buttonSave, global.doublebuffer, 0, 0, points[0][0], points[0][1], buttonSave->w, buttonSave->h);
		blit(buttonContinue, global.doublebuffer, 0, 0, points[1][0], points[1][1], buttonContinue->w, buttonContinue->h);
		if(global.vsync)
			vsync();
		show(true);
	}
	if(campaign)
		destroy_bitmap(missionDesc);
	destroy_bitmap(buttonContinue);
	destroy_bitmap(buttonSave);
}

void singlePlayerGame::reviewMissionDescription()
{
	int i;
	BITMAP *bg = createBitmap(map.getMapPreview(maxSizeX, maxSizeY)->w, map.getMapPreview(maxSizeX, maxSizeY)->h), *buttonContinue = loadBitmap("gfx/buttonContinue.tga");
	int points[2] = { SCREEN_W - buttonContinue->w, SCREEN_H - buttonContinue->h };
	blit(map.getMapPreview(maxSizeX, maxSizeY), bg, 0, 0, 0, 0, bg->w, bg->h);
	for(i = 0; i < objectives; ++i)
	{
		int myCol = makecol(0xFF, 0, 0);
		if(VM.getVar(18 + i))
			myCol = makecol(0, 0, 0xFF);
		circlefill(bg, (objectivePosXs[i] / map.getTilesize()) * map.getSmallSize(), (objectivePosYs[i] / map.getTilesize()) * map.getSmallSize(), map.getSmallSize(), myCol);
	}
	circlefill(bg, (player.getPosX() / map.getTilesize()) * map.getSmallSize(), (player.getPosY() / map.getTilesize()) * map.getSmallSize(), map.getSmallSize(), makecol(0, 0xFF, 0));

	blit(bg, missionDesc, 0, 0, SCREEN_W - maxSizeX + (maxSizeX - bg->w) / 2, (maxSizeY - bg->h) / 2, bg->w, bg->h);
	destroy_bitmap(bg);
	int posY = objectiveHeight;
	for(i = 0; i < objectives; ++i)
	{
		if(VM.getVar(18 + i))
			textout_right_ex(missionDesc, global.size12, "Success!", SCREEN_W, posY, makecol(0x00, 0xAA, 0x00), -1);
		posY += text_height(global.size12) + 3;
	}


	while(true)
	{
		if(timerCounter > 0)
		{
			if(keyStates[KEY_ENTER] || keyStates[global.keyUse] || (clickEvent() && mouse_x >= points[0] && mouse_y >= points[1]))
				break;
			--timerCounter;
		}
		blit(missionDesc, global.doublebuffer, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
		blit(buttonContinue, global.doublebuffer, 0, 0, points[0], points[1], buttonContinue->w, buttonContinue->h);
		//textout_centre_ex(global.doublebuffer, global.size20, "Press Enter to return to the map", 255, SCREEN_H - 2 - text_height(global.size20), 0, -1);
		if(global.vsync)
			vsync();
		show(true);
	}
	destroy_bitmap(buttonContinue);
}

void singlePlayerGame::addBullets(bulletRequest *bReq)
{
	if(bReq)
	{
		int j = 0;
		for(int i = 0; i < BULLETS[bReq->getType()]; ++i)
		{
			if(!global.complexSound || bReq->getTeam() == &player)
				bulletSounds[bReq->getType()].play(0, global.sfxVol);
			else
			{
				int destDir = (rad2deg(atan2(bReq->getPosX() - float(player.getPosX()), bReq->getPosY() - float(player.getPosY()))) * -1 + 128) & 0xFF;
				destDir = (destDir + 256 - player.getDirection()) & 0xFF;
				//If the bullet was fired from behind, multiplie with 0.8
				const float backShot = 0.8;
				int volume = global.sfxVol;
				int pan = 127;
				//Bullet was created on the right side of the player
				if(destDir < 128)
				{
					if(destDir > 64)
					{
						volume = int(float(volume) * backShot);
						pan = 127 + (destDir - 128) * -2;
					}
					else
					{
						pan = 127 + destDir * 2;
					}
				}
				//Bullet was created on the left side of the player
				else
				{
					if(destDir < 192)
					{
						volume = int(float(volume) * backShot);
						pan = 127 - (destDir - 128) * 2;
					}
					else
					{
						pan = 127 + (destDir - 256) * 2;
					}
				}
				pan &= 255;
				bulletSounds[bReq->getType()].play(0, volume, pan);
			}
			//play_sample(bulletSounds[bReq->getType()], global.sfxVol, 128, 1000, 0);
			for( ; j < MAX_BULLETS; ++j)
			{
				if(!bullets[j].isActive())
				{
					int spread[2] = { 1, -1 };
					int rd = rand();
					int direction;
					if(BULLETS[bReq->getType()] > 1)
					{
						direction = bReq->getDir() - (SPREAD[bReq->getType()] + 2) + (SPREAD[bReq->getType()] << 1) / BULLETS[bReq->getType()] * (i + 1);
					}
					else if(SPREAD[bReq->getType()])
						direction = bReq->getDir() + (rd % SPREAD[bReq->getType()]) * spread[rd & 1];
					else
						direction = bReq->getDir();
					bullets[j].init(bReq->getType(), direction, bReq->getPosX(), bReq->getPosY(), bReq->getTeam(), true);
					break;
				}
			}
		}
	}
}

void singlePlayerGame::explosion(bulletRequest bReq)
{
	int angle = 0;
	bulletSounds[bReq.getType()].play(0, global.sfxVol);
	//play_sample(bulletSounds[bReq.getType()], global.sfxVol, 128, 1000, 0);
	int j = 0;
	while(angle < 256)
	{
		for( ; j < MAX_BULLETS; ++j)
		{
			if(!bullets[j].isActive())
			{
				bullets[j].init(bReq.getType(), angle, bReq.getPosX(), bReq.getPosY(), 0, false);
				angle += SPLINTS[bReq.getType() - 2];
				break;
			}
		}
	}
}

bool singlePlayerGame::collides(int left1, int top1, int right1, int bottom1, int left2, int top2, int right2, int bottom2)
{
	bool collides[4] = { false, false, false, false };

	//Left side colliding
	if(left1 <= right2 && left1 >= left2)
	{
		collides[0] = true;
	}
	//Right side colliding
	if(right1 >= left2 && right1 <= right2)
	{
		collides[1] = true;
	}
	//Top side colliding
	if(top1 <= bottom2 && top1 >= top2)
	{
		collides[2] = true;
	}
	//Bottom side colliding
	if(bottom1 >= top2 && bottom1 <= bottom2)
	{
		collides[3] = true;
	}

	if(!(collides[0] || collides[1] || collides[2] || collides[3]))
		return false;

	//Check total collision for all 4 corners
	if(collides[0] && collides[2])
		return true;
	if(collides[0] && collides[3])
		return true;
	if(collides[1] && collides[2])
		return true;
	if(collides[1] && collides[3])
		return true;

	if(bottom1 - top1 > bottom2 - top2)
	{
		if((collides[0] || collides[1]) && top1 < top2 && bottom1 > bottom2)
			return true;
	}
	if(right1 - left1 > right2 - left2)
	{
		if((collides[2] || collides[3]) && left1 < left2 && right1 > right2)
			return true;
	}

	return false;
}
