/*
  Name: Unit.cpp
  Copyright: Polecatsoft 2005
  Game: Mercenary Soldier
  Author: Master Lenman
  Date: -
  Description: The unit class is derived from the Entity Class
    and is the parent class for all Units (Player, Enemies, Civilians)
*/

#include <allegro.h>
#include <libnet.h>
#include <string>
using namespace std;

#include "requests.h"
#include "ElementaryFunctions.h"
#include "Entity.h"
#include "Unit.h"
#include "Global.h"
#include "Errors.h"

moveRequest *Unit::getMoveRequest(int playerPosX, int playerPosY)
{
	return 0;
}

bulletRequest *Unit::getBulletRequest()
{
	return 0;
}

Unit::Unit() : Entity()
{
	maxLife = life = 0;
	maxArmor = armor = 0;
	armorType = 0;
	shootCooldown = 0;
	phase = 0;
	bloodPhase = 0;
	enemyOrCivilian = 0;
	sprites = new BITMAP*[MAXSPRITES];
	for(int i = 0; i < MAXSPRITES; ++i)
		sprites[i] = NULL;
	destroy = true;
}

bool Unit::isEnemy()
{
	return !enemyOrCivilian;
}

//void Unit::setCivilian()
//{
//	enemyOrCivilian = 1;
//}

void Unit::getHit(int bulletType)
{
	//Not getting hit is no longer possible
//	if(curState == STATE_DUCK && !(rand() % 5))
//		return;
	//No armor or armor piercing bullets
	if(PIERCE[bulletType] >= armorType)
	{
		life -= DAMAGE[bulletType];
	}
	//Light armor
	else if(armorType == 1)
	{
		int damageSub = int(float(DAMAGE[bulletType]) * 0.25);
		if(armor >= damageSub)
		{
			armor -= damageSub;
			life = life - DAMAGE[bulletType] + damageSub;
		}
		else
		{
			int damageLeft = DAMAGE[bulletType] - armor;
			armor = 0;
			life -= damageLeft;
		}
	}
	//Heavy armor blocks all damage if not pierced

	if(armor < 0)
		armor = 0;
	if(life < 0)
		life = 0;
}

void Unit::draw(int cameraX, int cameraY)
{
	if(!life)
	{
		phase = 0;
		if(bloodPhase == 0)
		{
			if(global.showBlood)
			{
				++bloodPhase;
			}
			else
			{
				//1 tick longer than it will take until the we will leave the game
				bloodPhase = 129;
			}
		}
		else
		{
			if(global.showBlood)
			{
				circlefill(global.doublebuffer, int(posX) - cameraX, int(posY) - cameraY, bloodPhase >> 2, makecol(0xFF, 0, 0));
				if(bloodPhase < (radius + 3) << 2)
					++bloodPhase;
			}
			else
			{
				if(bloodPhase < 0)
					return;
				--bloodPhase;
				if(!bloodPhase)
					bloodPhase = -1;
			}
		}
		curState = STATE_DEAD;
	}
	if(phase > 8)
		rotate_sprite_v_flip(global.doublebuffer, sprites[curState], int(posX) - cameraX - sprites[curState]->w / 2, int(posY) - cameraY - sprites[curState]->h / 2, itofix(direction + 128));
	else
		rotate_sprite(global.doublebuffer, sprites[curState], int(posX) - cameraX - sprites[curState]->w / 2, int(posY) - cameraY - sprites[curState]->h / 2, itofix(direction));
}

//Information stored in the map file are asigned here
void Unit::initialise(float initPosX, float initPosY, int initDir, int initState)
{
	posX = initPosX;
	posY = initPosY;
	direction = initDir;
	AIstate = initState;
}

//Information stored in the character set are asigned here
void Unit::load(float speed, int maxLife, int maxArmor, int armType, string spriteData, int trigDist, int weapon, int shootPause)
{
	int bla = 0;
	if(!sprites)//doConstructor
		sprites = new BITMAP*[MAXSPRITES];
	this->speed = speed;
	this->maxLife = life = maxLife;
	this->maxArmor = armor = maxArmor;
	this->triggerDist = trigDist;
	this->shootFreq = shootPause;
	armorType = armType;
	curWeapon = weapon;
	DATAFILE *datfile = load_datafile(spriteData.c_str());
	if(!datfile)
		errors::datafileError(spriteData);
	int i;
	for(i = 0; i < MAXSPRITES; ++i)
		sprites[i] = createDatafileBitmap(datfile, i);
	radius = *((char*)datfile[i].dat);
	unload_datafile(datfile);
}

//This will set the position stored in the request and destroy it afterwards
void Unit::takeRequests(moveRequest *mReq, bulletRequest *bReq)
{
	if(mReq)
	{
		posX = mReq->getNewPosX();
		posY = mReq->getNewPosY();

		delete mReq;
	}
	if(bReq)
	{
		delete bReq;
	}
}

void Unit::basetypeGet(Unit &basetype)
{
	destroy = false;
	for(int i = 0; i < MAXSPRITES; ++i)
		sprites[i] = basetype.getSpriteAddress(i);
	speed = basetype.getSpeed();
	maxLife = life = basetype.getMaxlife();
	maxArmor = armor = basetype.getMaxarmor();
	radius = basetype.getRadius();
	armorType = basetype.getArmortype();
	triggerDist = basetype.getTriggerDist();
	curWeapon = basetype.getWeapon();
	shootFreq = basetype.getShootFreq();
}

BITMAP *Unit::getSpriteAddress(int offset)
{
	return sprites[offset];
}

float Unit::getSpeed()
{
	return speed;
}

int Unit::getMaxarmor()
{
	return maxArmor;
}

int Unit::getArmortype()
{
	return armorType;
}

int Unit::getMaxlife()
{
	return maxLife;
}

bool Unit::isAlife()
{
	if(!life)
		return false;
	return true;
}

int Unit::getLifePercentage()
{
	return int(float(life) / (float(maxLife) / 100.0));
}

int Unit::getArmorPercentage()
{
	return int(float(armor) / (float(maxArmor) / 100.0));
}

void Unit::setState(int newState)
{
	curState = newState;
}

int Unit::getTriggerDist()
{
	return triggerDist;
}

int Unit::getWeapon()
{
	return curWeapon;
}

int Unit::getShootFreq()
{
	return shootFreq;
}

Unit::~Unit()
{
	for(int i = 0; i < MAXSPRITES; ++i)
	{
		if(!destroy)
			break;
		if(sprites[i])
			destroy_bitmap(sprites[i]);
	}
	delete [] sprites;
}
