/*
  Name: bullet.cpp
  Copyright: Polecatsoft 2005
  Game: Mercenary Soldier
  Author: Master Lenman
  Date: -
  Description: The bullet class is derived from the Entity class,
        every bullet shot in the game has an entity of this class
*/

#include <allegro.h>
#include <string>
#include <cmath>
using namespace std;

#include "Sound.h"
#include "global.h"
#include "requests.h"
#include "ElementaryFunctions.h"
#include "Entity.h"
#include "bullet.h"
#include "Errors.h"

BITMAP *bulletSprites[WEAPONTYPES] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
Sound bulletSounds[WEAPONTYPES];// = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };

void bullet::init(int initType, int dir, float initPosX, float initPosY, void *iTeam, bool immune)
{
	active = true;
	posX = initPosX;
	posY = initPosY;
	type = initType;
	direction = dir;
	moved = 0;
	team = iTeam;
//	if(immune)
//	{
//		immunity = 3;
//		//Grenades are immune for a longer time
//		if(type == 4)
//			immunity += 6;
//	}
//	else
//		immunity = 0;
}

moveRequest *bullet::update()
{
	moveRequest *mReq = NULL;
	if(active)
	{
		mReq = new moveRequest;
		float newX = sin(deg2rad(direction)) * SPEED[type];
		float newY = cos(deg2rad(direction)) * SPEED[type] * -1.0;

		int rights = RIGHT_MIDDLE;
		//If the bullet is a grenade, set the type to high
		if(type == 4)
			rights = RIGHT_HIGH;
		int scripts = 0;
		if(SPLINTS[type])
			scripts |= SCRIPT_EXPLODE;
		mReq->set(posX, posY, newX + posX, newY + posY, scripts, rights, bulletSprites[type]->h / 2);

		++moved;
		if(moved >= RANGE[type])
			active = false;
	/*	//Calculate distance travelled
		if(newX < 0)
			newX *= -1.0;
		if(newY < 0)
			newY *= -1.0;
		moved += (newX + newY) / SPEED[type];
		if(moved >= RANGE[type])
		{
			active = false;
		}*/
	}
//	if(immunity)
//		--immunity;
	return mReq;
}

bool bullet::returnRequest(moveRequest *mReq)
{
	if(mReq)
	{
		if(mReq->wasDeclined())
			active = false;
		else
		{
			posX = mReq->getNewPosX();
			posY = mReq->getNewPosY();
		}
		if(!active && SPLINTS[type])
			return 0;
		delete [] mReq;
	}
	return 1;
}

bullet::bullet() : Entity()
{
	type = -1;
	team = 0;
	moved = 0;
	for(int i = 0; i < WEAPONTYPES; ++i)
	{
		if(!bulletSprites[i])
		{
			//char(48) == '0', so i is added to get the paths to bullet1.tga, bullet2.tga...
			string path = string("gfx/bullet") + char(48 + i) + string(".tga");
			bulletSprites[i] = loadBitmap(path);
		}
		if(!bulletSounds[i].isActive())
		{
			string path = string("sfx/weapon") + char(48 + i) + string(".wav");
			bulletSounds[i].load(path);
//			bulletSounds[i] = load_sample(path.c_str());
//			if(!bulletSounds[i])
//				errors::sampleLoadError(path);
		}
	}
	active = false;
}

bullet::~bullet()
{
	for(int i = 0; i < WEAPONTYPES; ++i)
	{
		bulletSounds[i].destroy();
		//if(bulletSounds[i])
		//	destroy_sample(bulletSounds[i]);
		//bulletSounds[i] = NULL;
	}
}

void bullet::draw(int cameraX, int cameraY)
{
	if(active)
	{
		rotate_sprite(global.doublebuffer, bulletSprites[type], int(posX) - cameraX - bulletSprites[curState]->w / 2, int(posY) - cameraY - bulletSprites[type]->h / 2, itofix(direction));
	}
}

bool bullet::isActive()
{
	return active;
}

int bullet::getType()
{
	return type;
}

void *bullet::getTeam()
{
	return team;
}

//bool bullet::isImmune()
//{
//	if(immunity)
//		return true;
//	return false;
//}
