/*
  Name: menues.cpp
  Copyright: Polecatsoft 2005
  Game: Mercenary Soldier
  Author: Master Lenman
  Date: -
  Description: All menu functions are stored in this file
*/

#include <allegro.h>
#include <libnet.h>
#include <string>
#include <fstream>
using namespace std;

#include "ElementaryFunctions.h"
#include "Global.h"
#include "menues.h"
#include "Errors.h"
#include "keyStatesClass.h"
#include "mouse.h"

int menu::main(const int startValue)
{
	const int BUTTONS = 5;
	string labels[BUTTONS] = { "New Game", "Load Game", "Multiplayer", "Options", "Exit" };
	bool states[BUTTONS] = {    true,       true,       true,         true,      true };
	if(!savegameExists())
		states[1] = false;
	return stdMenu(startValue, BUTTONS, labels, states);
}

int menu::newGameMenu()
{
	const int BUTTONS = 3;
	string labels[BUTTONS] = { "Start New Game", "Special Maps", "Cancel" };
	bool states[BUTTONS] = {         true,             true,           true     };
	int retCatch = stdMenu(0, BUTTONS, labels, states);

	if(retCatch == 1)
	{
		const int BUTTONS2 = 3;
		string labels2[BUTTONS2] = { "Shooting Range", "Tutorial", "Cancel" };
		bool states2[BUTTONS] = {           true,			  true,       true    };
		retCatch = BUTTONS + stdMenu(0, BUTTONS2, labels2, states2);
		if(retCatch == BUTTONS + BUTTONS2 - 1)
			retCatch = newGameMenu();
	}

	return retCatch;
}

int menu::serverClient()
{
	const int BUTTONS = 3;
	string labels[BUTTONS] = { "Server", "Client", "Cancel" };
	bool states[BUTTONS] = {          true,    true,     true     };
	return stdMenu(0, BUTTONS, labels, states);
}

int menu::inGameMenu(BITMAP *background)
{
	BITMAP *button = loadBitmap("gfx/ingamebutton.tga");
	BITMAP *buttonpressed = loadBitmap("gfx/ingamebuttonpressed.tga");

	int borderHeight = 250;
	const int BUTTONS = 4;
	string labels[BUTTONS] = { "Continue", "Mission description", "Options", "Quit to main menu" };
	bool states[BUTTONS] = { true, true, true, true };
	int retVal = stdMenu(0, BUTTONS, labels, states, global.size12, borderHeight, button, buttonpressed, background, makecol(0xAA, 0xAA, 0xAA), makecol(0xFF, 0xFF, 0xFF));
	destroy_bitmap(button);
	destroy_bitmap(buttonpressed);

	return retVal;
}

bool menu::options(const bool fromWhere)
{
	int curSelection = 0;
	int posYPlus = text_height(global.size20) + 3,
		smallPosYPlus = text_height(global.size12) + 3;
	const int OPTIONS = 9;
	const char *names[OPTIONS] = { "Display mode", "Color depth", "VSync", "Complex SFX", "SFX volume", "Music volume", "Controls", "Exit saving changes", "Exit discarding changes" };
	const char *descriptions[OPTIONS] = { "Should the game be started in windowed or fullscreen mode?", "How many bits per pixel should be used?", "Should the game be synced with your monitor?", "Should 2D Sound effects be used?", "Volume of sound effects", "Volume of background music", "Set controls", "Return to main menu saving changes you made", "Return to main menu discarding all changes you made" };
	// 0 = 2 possibilites, 1 = volume control, 2 = button
	const int type[OPTIONS] = { 0, 0, 0, 0, 1, 1, 2, 2, 2 };

	const char *possiblities[4][2] = {
		{ "Windowed", "Fullscreen" },
		{ "32", "16" },
		{ "On", "Off" },
		{ "On", "Off" },
	};
	bool selTable[4] = { global.curmode == 1, global.depth == 16, global.vsync == 0, global.complexSound == false };

	//Needed for mouse routines
	bool useMouse = false;
	int posY[OPTIONS + 1];
	posY[OPTIONS] = SCREEN_H;
	
	int i;
	globals temp = global;
	unsigned char *useAddress[OPTIONS - 3] = { NULL, NULL, NULL, NULL, &temp.sfxVol, &temp.musicVol };
	bool run = true;
	int initPosY = (SCREEN_H - OPTIONS * posYPlus - smallPosYPlus) >> 1;
	if(!fromWhere)
		initPosY = (SCREEN_H - (OPTIONS - 2) * posYPlus - smallPosYPlus) >> 1;
	while(run)
	{
		if(timerCounter > 0)
		{
			for(i = 0; i < OPTIONS; ++i)
			{
				if(!useMouse)
				    break;
				if(mouse_y > posY[i] && mouse_y < posY[i + 1])
				{
				    curSelection = i;
				    break;
				}
			}
			if(keyStates[KEY_UP] || keyStates[global.keyUp])
			{
				if(!curSelection)
					curSelection = OPTIONS - 1;
				else
					--curSelection;
				if(!fromWhere && curSelection == 1)
					--curSelection;
			}
			if(keyStates[KEY_DOWN] || keyStates[global.keyDown])
			{
				++curSelection %= OPTIONS;
				if(!fromWhere && curSelection == 1)
					++curSelection;
			}
			if(keyStates[KEY_ENTER] || keyStates[global.keyUse] || clickEvent())
			{
				switch(curSelection)
				{
					//controls
				case OPTIONS - 3:
					keyChange(temp);
					break;
					//Exit
				case OPTIONS - 2:
					run = false;
					break;
					//Reset
				case OPTIONS - 1:
					temp = global;
					run = false;
					break;
				}
			}
			if(keyStates[KEY_LEFT] || keyStates[global.keyLeft])
			{
				switch(curSelection)
				{
					//Display mode
				case 0:
					temp.curmode = 2;
					break;
					//Color depth
				case 1:
					temp.depth = 32;
					break;
					//Vsync
				case 2:
					temp.vsync = true;
					break;
				case 3:
					temp.complexSound = true;
					break;
				}
			}
			if(keyStates[KEY_RIGHT] || keyStates[global.keyRight])
			{
				switch(curSelection)
				{
					//Display mode
				case 0:
					temp.curmode = 1;
					break;
					//Color depth
				case 1:
					temp.depth = 16;
					break;
					//Vsync
				case 2:
					temp.vsync = false;
					break;
				case 3:
					temp.complexSound = false;
					break;
				}
			}
			if(key[KEY_RIGHT] || key[global.keyRight])
			{
				switch(curSelection)
				{
					//Sfx Vol +
				case 4:
					if(temp.sfxVol < 0xFF)
						++temp.sfxVol;
					break;
					//Music Vol +
				case 5:
					if(temp.musicVol < 0xFF)
						++temp.musicVol;
					break;
				}
			}
			if(key[KEY_LEFT] || key[global.keyLeft])
			{
				switch(curSelection)
				{
					//Sfx Vol -
				case 4:
					if(temp.sfxVol > 0)
						--temp.sfxVol;
					break;
					//Music Vol -
				case 5:
					if(temp.musicVol > 0)
						--temp.musicVol;
					break;
				}
			}
			if(mouseZDiff)
			{
				switch(curSelection)
				{
					//Sfx Vol -
				case 4:
					temp.sfxVol += mouseZDiff;
					break;
					//Music Vol -
				case 5:
					temp.musicVol += mouseZDiff;
					break;
				}
			}
			selTable[0] = temp.curmode == 1;
			selTable[1] = temp.depth == 16;
			selTable[2] = temp.vsync == 0;
			selTable[3] = temp.complexSound == false;
			--timerCounter;
		}
		int drawPosY = initPosY;

		clear(global.doublebuffer);
		blit(global.mainmenu, global.doublebuffer, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
		for(i = 0; i < OPTIONS; ++i)
		{
			posY[i] = drawPosY;
			if(!fromWhere && i == 1)
				continue;
			switch(type[i])
			{
				//Selector
			case 0:
				int colors[3];
				if(curSelection == i)
				{
					//colors[0] = makecol(0xD0, 0xD0, 0xD0);
					colors[0] = makecol(0xFF, 0xFF, 0xFF);
					colors[1] = makecol(0x20, 0x20, 0x20);
					colors[2] = colors[0];
					textout_centre_ex(global.doublebuffer, global.size12, descriptions[i], SCREEN_W >> 1, drawPosY + posYPlus, makecol(0xFF, 0xFF, 0xFF), -1);
				}
				else
				{
					colors[0] = makecol(0x70, 0x70, 0x70);
					colors[1] = makecol(0x20, 0x20, 0x20);
					colors[2] = makecol(0x00, 0x00, 0x00);
				}
				if(selTable[i])
					colors[0] ^= colors[1] ^= colors[0] ^= colors[1];
				textout_centre_ex(global.doublebuffer, global.size20, possiblities[i][0], SCREEN_W >> 1, drawPosY, colors[0], -1);
				textout_ex(global.doublebuffer, global.size20, possiblities[i][1], ((SCREEN_W + text_length(global.size20, possiblities[i][0])) >> 1) + 26, drawPosY, colors[1], -1);
				textout_right_ex(global.doublebuffer, global.size20, names[i], ((SCREEN_W - text_length(global.size20, possiblities[i][0])) >> 1) - 26, drawPosY, colors[2], -1);
				drawPosY += posYPlus;
				if(curSelection == i)
					drawPosY += smallPosYPlus;
				break;
				//Volume adjust
			case 1:
				if(curSelection == i)
				{
					textout_right_ex(global.doublebuffer, global.size20, names[i], SCREEN_W >> 1, drawPosY, makecol(0xFF, 0xFF, 0xFF), -1);
					textprintf_ex(global.doublebuffer, global.size20, (SCREEN_W >> 1) + 5, drawPosY, makecol(0xFF, 0xFF, 0xFF), -1, " - %i%%",  int(*useAddress[i] / 255.0 * 100.0));
					drawPosY += posYPlus;
					textout_centre_ex(global.doublebuffer, global.size12, descriptions[i], SCREEN_W >> 1, drawPosY, makecol(0xFF, 0xFF, 0xFF), -1);
					drawPosY += smallPosYPlus;
				}
				else
				{
					textout_right_ex(global.doublebuffer, global.size20, names[i], SCREEN_W >> 1, drawPosY, 0, -1);
					textprintf_ex(global.doublebuffer, global.size20, (SCREEN_W >> 1) + 5, drawPosY, 0, -1, " - %i%%", int(*useAddress[i] / 255.0 * 100.0));
					drawPosY += posYPlus;
				}
				break;
			case 2:
				//Button
				if(curSelection == i)
				{
					textout_centre_ex(global.doublebuffer, global.size20, names[i], SCREEN_W >> 1, drawPosY, makecol(0xFF, 0xFF, 0xFF), -1);
					drawPosY += posYPlus;
					textout_centre_ex(global.doublebuffer, global.size12, descriptions[i], SCREEN_W >> 1, drawPosY, makecol(0xFF, 0xFF, 0xFF), -1);
					drawPosY += smallPosYPlus;
				}
				else
				{
					textout_centre_ex(global.doublebuffer, global.size20, names[i], SCREEN_W >> 1, drawPosY, 0, -1);
					drawPosY += posYPlus;
				}
				break;
			}
		}
		if(global.vsync)
			vsync();
		useMouse = true;
		show(true);
	}
	if(global.curmode != temp.curmode)
		setMode(temp.curmode);

	bool toReturn = false;
	if(global.depth != temp.depth || global.useVideoRAM != temp.useVideoRAM)
		toReturn = true;

	global = temp;
	return toReturn;
}

void menu::keyChange(globals &temp)
{
	const int NUMRESERVEDKEYS = 9;
	const int RESERVEDKEYS[NUMRESERVEDKEYS] = { KEY_1, KEY_2, KEY_3, KEY_4, KEY_5, KEY_F9, KEY_F10, KEY_F11, KEY_F12 };
	int curSelection = 0;
	int posYPlus = text_height(global.size20) + 3,
		smallPosYPlus = text_height(global.size12) + 3;
	const int OPTIONS = 12;
	const char *names[OPTIONS] = { "Move forward", "Move backward", "Strafe left", "Strafe right", "Shoot", "Use", "Select next weapon", "Select previous weapon", "Show aiming help", "Review mission description", "View old messages", "Finished" };
	const char *descriptions[OPTIONS] = { "moving forward", "moving backward", "moving to the left", "moving to the right", "firing the selected weapon", "using things like health stations", "selecting the next weapon", "selecting the previous weapon", "showing a line indicating the way your bullets will fly", "showing the description of your current mission again", "showing an archive of received messages", "Return to the other option" };
	// 0 = key, 1 = button
	const int type[OPTIONS] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 };
	//This is needed for mouse routines
	int posY[OPTIONS + 1];
	posY[OPTIONS] = SCREEN_H;
	bool useMouse = false;

	int *addresses[OPTIONS - 1] = { &temp.keyUp, &temp.keyDown, &temp.keyLeft, &temp.keyRight, &temp.keyShoot,
		&temp.keyUse, &temp.keyNextweapon, &temp.keyPreviousweapon, &temp.keyAim, &temp.keyMission, &temp.keyMessages };

	int i;
	bool run = true;
	int initPosY = (SCREEN_H - OPTIONS * posYPlus - smallPosYPlus) >> 1;
	//Buffer for strings
	string toShow;
	while(run)
	{
		if(timerCounter > 0)
		{
			for(i = 0; i < OPTIONS; ++i)
			{
				if(!useMouse)
				    break;
				if(mouse_y > posY[i] && mouse_y < posY[i + 1])
				{
				    curSelection = i;
				    break;
				}
			}
			if(keyStates[KEY_UP] || keyStates[global.keyUp])
			{
				if(!curSelection)
					curSelection = OPTIONS - 1;
				else
					--curSelection;
			}
			if(keyStates[KEY_DOWN] || keyStates[global.keyDown])
			{
				++curSelection %= OPTIONS;
			}
			if(keyStates[KEY_ENTER] || keyStates[global.keyUse] || clickEvent())
			{
				//Select a new key
				if(curSelection == OPTIONS - 1)
					run = false;
				else
				{
					string text = string("Please press the key you would like to use for ") + string(descriptions[curSelection]);
					string text2 = string("Current key assigned is ") + keyNames[*addresses[curSelection]];
					string error;
					int errorShowTime = 0;
					int useCountdown = 16;
					while(true)
					{
						if(timerCounter > 0)
						{
							if(errorShowTime)
								--errorShowTime;
							if(keyStates[KEY_ESC])
								break;
							bool found = false;
							for(int h = 0; h < KEY_MAX; ++h)
							{
								if(h == KEY_ENTER)
									continue;
								if(useCountdown)
								{
									if(h == global.keyUse)
									{
										--useCountdown;
									    continue;
									}
								}
								if(key[h])
								{
									bool failure = false;
									int g;
									for(g = 0; g < OPTIONS - 1; ++g)
									{
										if(g == curSelection)
											continue;
										if(*addresses[g] == h)
										{
											error = string("This key is already used for ") + string(descriptions[g]);
											errorShowTime = 80;
											failure = true;
											break;
										}
									}
									for(g = 0; g < NUMRESERVEDKEYS; ++g)
									{
										if(h == RESERVEDKEYS[g])
										{
											error = keyNames[h] + string(" is a reserved key");
											errorShowTime = 80;
											failure = true;
											break;
										}
									}
									if(!failure)
									{
										*addresses[curSelection] = h;
										found = true;
										break;
									}
								}
							}
							if(found)
								break;
							--timerCounter;
						}
						clear(global.doublebuffer);
						blit(global.mainmenu, global.doublebuffer, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
						textout_centre_ex(global.doublebuffer, global.size12, "Press Esc to cancel", SCREEN_W >> 1, 280, makecol(0xFF, 0xFF, 0xFF), -1);
						textout_centre_ex(global.doublebuffer, global.size12, text.c_str(), SCREEN_W >> 1, 300, makecol(0xFF, 0xFF, 0xFF), -1);
						textout_centre_ex(global.doublebuffer, global.size12, text2.c_str(), SCREEN_W >> 1, 320, makecol(0xFF, 0xFF, 0xFF), -1);
						if(errorShowTime)
							textout_centre_ex(global.doublebuffer, global.size12bold, error.c_str(), SCREEN_W >> 1, 350, makecol(0xFF, 0x00, 0x00), -1);
						show(true);
					}
				}
			}
			--timerCounter;
		}
		int drawPosY = initPosY;

		clear(global.doublebuffer);
		blit(global.mainmenu, global.doublebuffer, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
		for(i = 0; i < OPTIONS; ++i)
		{
			posY[i] = drawPosY;
			switch(type[i])
			{
			case 0:
				if(curSelection == i)
				{
					toShow = string(names[i]) + string(" (") + keyNames[*addresses[i]] + string(")");
					textout_centre_ex(global.doublebuffer, global.size20, toShow.c_str() , SCREEN_W >> 1, drawPosY, makecol(0xFF, 0xFF, 0xFF), -1);
					drawPosY += posYPlus;

					toShow = string("Change key for ") + string(descriptions[i]);
					textout_centre_ex(global.doublebuffer, global.size12, toShow.c_str(), SCREEN_W >> 1, drawPosY, makecol(0xFF, 0xFF, 0xFF), -1);
					drawPosY += smallPosYPlus;
				}
				else
				{
					toShow = string(names[i]) + string(" (") + keyNames[*addresses[i]] + string(")");
					textout_centre_ex(global.doublebuffer, global.size20, toShow.c_str(), SCREEN_W >> 1, drawPosY, 0, -1);
					drawPosY += posYPlus;
				}
				break;
			case 1:
				//Button
				if(curSelection == i)
				{
					textout_centre_ex(global.doublebuffer, global.size20, names[i], SCREEN_W >> 1, drawPosY, makecol(0xFF, 0xFF, 0xFF), -1);
					drawPosY += posYPlus;
					textout_centre_ex(global.doublebuffer, global.size12, descriptions[i], SCREEN_W >> 1, drawPosY, makecol(0xFF, 0xFF, 0xFF), -1);
					drawPosY += smallPosYPlus;
				}
				else
				{
					textout_centre_ex(global.doublebuffer, global.size20, names[i], SCREEN_W >> 1, drawPosY, 0, -1);
					drawPosY += posYPlus;
				}
				break;
			}
		}
		if(global.vsync)
			vsync();
		useMouse = true;
		show(true);
	}
}

int menu::savegameSelect(bool grayEmpty)
{
	//Quit if no entry is true
	if(grayEmpty && !savegameExists())
		return -1;
	const int BUTTONS = 5;
	string labels[BUTTONS] = { "Autosave: ", "Slot 1: ", "Slot 2: ", "Slot 3: ", "Cancel" };
	bool states[BUTTONS] = {   false,      true,   true,   true,   true     };
	if(grayEmpty && fileExists(getSavegamePath(0)))
		states[0] = true;
	for(int i = 0; i < 4; ++i)
	{
		string temp = getSavegamePath(i);
		if(fileExists(temp))
		{
			ifstream savegame(temp.c_str());
			getline(savegame, temp);
			savegame.close();
			temp = string("maps/") + temp + string(".msd");
			ifstream missionDescription(temp.c_str());
			if(missionDescription.fail())
			{
				labels[i] += "Unknown";
				continue;
			}
			do getline(missionDescription, temp); while(temp[0] == '-');
			if(i && false)
				labels[i] = temp;
			else
			    labels[i] += temp;
		}
		else
		{
			labels[i] += "Empty";
			if(grayEmpty)
				states[i] = false;
		}
	}
	//If the load menu is shown, display the Autosave
	int mod = 0;
	//If the save menu is shown, do not display it
	if(!grayEmpty)
		mod = 1;
	int retCatch = stdMenu(0, BUTTONS - mod, &(labels[mod]), &(states[mod])) + mod;

	if(retCatch == BUTTONS - 1)
		retCatch = -1;
	return retCatch;
}

bool menu::savegameExists()
{
	bool quit = false;
	for(int i = 0; i < 4; ++i)
		if(fileExists(getSavegamePath(i)))
			quit = true;
	return quit;
}

int menu::stdMenu(const int startValue, const int BUTTONS, string *labels, bool *states, FONT *toUse, int borders, BITMAP *button, BITMAP *buttonpressed, BITMAP *background, int col_standard, int col_selected, int col_disabled)
{
	int i;
	int color_selected = makecol(0x00, 0x00, 0x00),
		color_disabled = makecol(0xFF, 0xFF, 0xFF),
		color_standard = makecol(0x99, 0x99, 0x99);
	if(col_selected > -1)
	    color_selected = col_selected;
	if(col_disabled > -1)
	    color_disabled = col_disabled;
	if(col_standard > -1)
	    color_standard = col_standard;
	bool hasActive = false;
	for(i = 0; i < BUTTONS; ++i)
	{
		if(states[i])
		{
			hasActive = true;
			break;
		}
	}
	if(!hasActive)
		states[BUTTONS - 1] = true;
	bool destroyButton = false, destroyButtonPressed = false;
	if(!background)
		background = global.mainmenu;
	if(!button)
	{
		button = loadBitmap("gfx/button.tga");
		destroyButton = true;
	}
	if(!buttonpressed)
	{
		buttonpressed = loadBitmap("gfx/buttonpressed.tga");
		destroyButtonPressed = true;
	}

	int curSelection = startValue;
	while(!states[curSelection])
		++curSelection;

	int *posY = new int[BUTTONS];
	//int space = (height - (borders * 2) - button->h * BUTTONS) / (BUTTONS - 1);
	int space = SCREEN_H - borders * 2;
	int singleSpace = space / BUTTONS;
	for(i = 0; i < BUTTONS; ++i)
	{
		posY[i] = borders + i * singleSpace + (singleSpace >> 1) - (button->h >> 1);
		//posY[i] = (borders >> 1) + i * space + i * button->h;
		int twidth = text_length(toUse, labels[i].c_str());
		if(twidth > button->w - 6)
		{
			do
			{
				labels[i].erase(labels[i].length() - 1, 1);
			} while(text_length(toUse, (labels[i] + "...").c_str()) > button->w - 6 || labels[i][labels[i].length() - 1] == ' ');
			labels[i].append(3, '.');
		}
	}
	int fontHeight = text_height(toUse);

	keyStates[KEY_UP];
	keyStates[KEY_DOWN];
	space = true;
	while(space)
	{
		if(timerCounter > 0)
		{
			for(i = 0; i < BUTTONS; ++i)
			{
				if(!states[i] || curSelection == i)
				    continue;
				if(mouse_y > posY[i] && mouse_y < posY[i] + button->h && mouse_x > ((SCREEN_W - button->w) >> 1) && mouse_x < ((SCREEN_W + button->w) >> 1))
				{
				    curSelection = i;
					break;
				}
			}
			if(keyStates[KEY_UP] || keyStates[global.keyUp])
			{
				if(!curSelection)
					curSelection = BUTTONS - 1;
				else
					--curSelection;
				while(!states[curSelection])
				{
					if(!curSelection)
						curSelection = BUTTONS - 1;
					else
						--curSelection;
				}
			}
			if(keyStates[KEY_DOWN] || keyStates[global.keyDown])
			{
				++curSelection %= BUTTONS;
				while(!states[curSelection])
				{
					++curSelection %= BUTTONS;
				}
			}
			if(keyStates[KEY_ENTER] || keyStates[global.keyUse] || clickEvent())
				space = false;
			--timerCounter;
		}
		blit(background, global.doublebuffer, 0, 0, 0, 0, background->w, background->h);
		for(i = 0; i < BUTTONS; ++i)
		{
			if(i == curSelection)
			{
				masked_blit(button, global.doublebuffer, 0, 0, (SCREEN_W - buttonpressed->w) >> 1, posY[i], buttonpressed->w, buttonpressed->h);
				textout_centre_ex(global.doublebuffer, toUse, labels[i].c_str(), SCREEN_W >> 1, posY[i] + ((button->h - fontHeight) >> 1), color_selected, -1);
			}
			else
			{
				masked_blit(buttonpressed, global.doublebuffer, 0, 0, (SCREEN_W - button->w) >> 1, posY[i], button->w, button->h);
				if(states[i])
					textout_centre_ex(global.doublebuffer, toUse, labels[i].c_str(), SCREEN_W >> 1, posY[i] + ((button->h - fontHeight) >> 1), color_standard, -1);
				else
					textout_centre_ex(global.doublebuffer, toUse, labels[i].c_str(), SCREEN_W >> 1, posY[i] + ((button->h - fontHeight) >> 1), color_disabled, -1);
			}
		}
		if(global.vsync)
			vsync();
		drawMouse();
		show(true);
	}

	delete [] posY;
	if(destroyButton)
		destroy_bitmap(buttonpressed);
	if(destroyButtonPressed)
		destroy_bitmap(button);
	return curSelection;
}

int menu::getDifficulty()
{
	const int BUTTONS = 3;
	string labels[BUTTONS] = { "Easy", "Medium", "Hard" };
	bool states[BUTTONS] = {         true,   true,     true };
	return stdMenu(0, BUTTONS, labels, states);
}

int menu::skipTutorial()
{
	const int BUTTONS = 2;
	string labels[BUTTONS] = { "Start tutorial", "Skip tutorial" };
	bool states[BUTTONS] = {         true,             true };
	return stdMenu(0, BUTTONS, labels, states);
}

void menu::intro()
{
	DATAFILE *datfile = load_datafile("data/intro.dat");
	if(!datfile)
		errors::datafileError("data/intro.dat");
	BITMAP *fakeBuffer = global.doublebuffer;
	if(global.useVideoRAM)
		fakeBuffer = create_bitmap(SCREEN_W, SCREEN_H);
	BITMAP *bg = (BITMAP*)datfile[0].dat,
		*wolf = (BITMAP*)datfile[1].dat,
		*ALLogo = (BITMAP*)datfile[3].dat;
	SAMPLE *bang = (SAMPLE*)datfile[2].dat;

	double wwidth = 0.0, wheight = 0.0;
	int countdown = 64;
	const double heightPlus = double(wolf->h) / double(countdown);
	const double widthPlus = double(wolf->w) / double(countdown);
	countdown += 48;
	int textposx = (SCREEN_W - text_length(global.size20, "A Polecatsoft game")) >> 1,
		textposy = SCREEN_H - 140 - text_height(global.size20);
	string text = "A";
	int textcolor = 0;
	int curSkip = 0;
	bool running = true;
	while(running)
	{
		while(timerCounter > 0)
		{
			if(keyStates.anyKey() && (!global.firstTime || textcolor >= 250))
				running = false;
			wheight += heightPlus;
			if(int(wheight) > wolf->h)
				wheight = double(wolf->h);
			wwidth += widthPlus;
			if(int(wwidth) > wolf->w)
				wwidth = double(wolf->w);

			if(countdown >= -14)
				--countdown;
			if(countdown == -15 && textcolor + 3 <= 255)
				textcolor += 3;
			if(countdown == 16)
				text += " Polecatsoft";
			if(!countdown)
				text += " game";
			if(countdown <= 48 && !(countdown & 15))
				play_sample(bang, global.sfxVol, 127, 1000, 0);
			--timerCounter;
			++curSkip;
			if(curSkip >= global.frameskip)
				timerCounter = 0;
		}
		clear(fakeBuffer);
		blit(bg, fakeBuffer, 0, 0, 0, 0, bg->w, bg->h);
		//masked_stretch_blit(BITMAP *source, BITMAP *dest, int source_x, source_y, source_w, source_h, int dest_x, dest_y, dest_w, dest_h);
		masked_stretch_blit(wolf, fakeBuffer, 0, 0, wolf->w, wolf->h, (SCREEN_W >> 1) - int(wwidth / 2.0), (SCREEN_H >> 1) - int(wheight / 2.0), int(wwidth), int(wheight));
		if(countdown <= 32)
			textout_ex(fakeBuffer, global.size20, text.c_str(), textposx, textposy, makecol(255, 255, 255), -1);
		if(countdown == -15)
		{
			blit(ALLogo, fakeBuffer, 0, 0, 3, SCREEN_H - ALLogo->h - 3, ALLogo->w, ALLogo->h);
			textout_centre_ex(fakeBuffer, global.size20, "www.Polecatsoft.tk", SCREEN_W >> 1, textposy + text_height(global.size20) + 7, makecol(textcolor, textcolor, textcolor), -1);
			textout_right_ex(fakeBuffer, global.size12, allegro_id, SCREEN_W - 3, SCREEN_H - 6 - 2 * text_height(global.size12), 0, -1);
			textout_right_ex(fakeBuffer, global.size12, VERSIONSTRING.c_str(), SCREEN_W - 3, SCREEN_H - 3 - text_height(global.size12), 0, -1);
		}
		if(global.vsync)
			vsync();
		show(true, fakeBuffer);
		curSkip = 0;
	}

	if(global.useVideoRAM)
		destroy_bitmap(fakeBuffer);
	unload_datafile(datfile);
}

void menu::gameOver()
{/*
	BITMAP *text = loadBitmap("gfx/gameover.tga");
	BITMAP *bg = createBitmap(SCREEN_W, SCREEN_H);
	blit(global.doublebuffer, bg, 0, 0, 0, 0, SCREEN_W, SCREEN_H);

	int countdown = 16;
	while(true)
	{
		if(timerCounter > 0)
		{
			if(!countdown)
			{
				if(keyStates.anyKey())
					break;
			}
			else
				--countdown;
			--timerCounter;
		}
		clear(global.doublebuffer);
		blit(bg, global.doublebuffer, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
		masked_blit(text, global.doublebuffer, 0, 0, (SCREEN_W - text->w) / 2, (SCREEN_H - text->h) / 2, text->w, text->h);
		if(global.vsync)
			vsync();
		show(true);
	}

	destroy_bitmap(text);
	destroy_bitmap(bg);*/
}
