/*
  Name: messageClass.cpp
  Copyright: Polecatsoft 2005
  Game: Mercenary Soldier
  Author: Master Lenman
  Date: -
  Description: Message class - this class manages and stores messages revieved from
	  scripts, other players and the game itself.
*/

#include <allegro.h>
#include <libnet.h>
#include <string>
#include <sstream>
using namespace std;

#include "ElementaryFunctions.h"
#include "messageClass.h"
#include "keyStatesClass.h"
#include "Global.h"

void message::poll()
{
	if(countdown)
		--countdown;
	if(!countdown)
		displayed = true;
}

message::message()
{
	displayed = false;
	countdown = 0;
}
void message::setCtd(int ctd)
{
	countdown = ctd;
}

void message::draw(int posX, int posY, BITMAP *dest, FONT *useFont)
{
	if(!empty())
	{
		textout_ex(dest, useFont, c_str(), posX, posY, 0, -1);
	}
}

void message::erase()
{
	string::erase();
	displayed = false;
}

bool message::isOld()
{
	return displayed;
}

void messageClass::sendText(string text)
{
	if(recBuffer[recBuffer.length() - 1] != ' ')
		recBuffer.append(1, ' ');
	recBuffer.append(text);
}

void messageClass::sendInt(int value)
{
	stringstream s;
	s << value;
	if(recBuffer[recBuffer.length() - 1] != ' ')
		recBuffer.append(1, ' ');
	recBuffer.append(s.str());
}

void messageClass::output()
{
	if(used < size)
		++used;
	for(int i = size - 2; i > 0; --i)
	{
		displayedMessages[i] = displayedMessages[i - 1];
	}
	displayedMessages[0].erase();
	displayedMessages[0] = recBuffer;
	recBuffer.erase();
}

messageClass::messageClass(int size, int maxVis, int stdX, FONT *mscFont, FONT *bldFont, BITMAP *drawTo)
{
	if(size)
		displayedMessages = new message[size];
	else
		displayedMessages = 0;

	this->size = size;
	used = 0;

	maxVisible = maxVis;

	stdFont = mscFont;
	boldFont = bldFont;
	dest = drawTo;

	posX = stdX;

	plus = text_height(stdFont) + 2;

	recBuffer.setCtd(128);
}

messageClass::~messageClass()
{
	if(displayedMessages)
		delete [] displayedMessages;
}

void messageClass::draw(int initPosY, bool forceOutput)
{
	if(!used)
		return;

	bool quit = true;
	int max = maxVisible;
	if(used < maxVisible)
		max = used;

	int i;
	for(i = 0; i < max; ++i)
	{
		if(!displayedMessages[i].isOld())
		{
			quit = false;
			break;
		}
	}
	if(quit && !forceOutput)
		return;

	int posY = initPosY;
	int maxX = 0;
	for(i = 0; i < max; ++i)
	{
		int temp;
		if(displayedMessages[i].isOld())
			temp = text_length(stdFont, displayedMessages[i].c_str()) + posX;
		else
			temp = text_length(boldFont, displayedMessages[i].c_str()) + posX;
		if(temp > maxX)
			maxX = temp;
	}
	maxX += 2;
	rectfill(dest, 0, initPosY - 2, maxX, initPosY + plus * max, makecol(0xFF, 0xFF, 0xFF));
	for(i = 0; i < max; ++i)
	{
		if(!displayedMessages[i].isOld())
			displayedMessages[i].draw(posX, posY, dest, boldFont);
		else
			displayedMessages[i].draw(posX, posY, dest, stdFont);
		posY += plus;
	}
}

void messageClass::poll()
{
	for(int i = 0; i < used; ++i)
		displayedMessages[i].poll();
}

void messageClass::showArchive()
{
	if(!used)
		return;
	keyStates.anyKey();
	while(true)
	{
		if(timerCounter > 0)
		{
			if(keyStates.anyKey())
				break;
			--timerCounter;
		}
		clear_to_color(dest, makecol(0xFF, 0xFF, 0xFF));
		textout_centre_ex(dest, boldFont, "Press any key to return to game.", SCREEN_W >> 1, 5, 0, -1);
		textout_centre_ex(dest, boldFont, "You received the following messages:", SCREEN_W >> 1, 5 + plus, 0, -1);
		for(int i = 0, posY = plus * 3 + 5; i < used && posY + plus < SCREEN_H; ++i, posY += plus)
			displayedMessages[i].draw(8, posY, dest, stdFont);

		if(global.vsync)
			vsync();
		show(false, dest);
	}
	for(int i = 0; i < used; ++i)
	{
		while(!displayedMessages[i].isOld())
			displayedMessages[i].poll();
	}
}
