/*
  Name: playerUnit.cpp
  Copyright: Polecatsoft 2005
  Game: Mercenary Soldier
  Author: Master Lenman
  Date: -
  Description: The playerUnit class is derived from the Unit Class.
	  For each player in the game, a new instance is created.
*/

#include <allegro.h>
#include <libnet.h>
#include <string>
#include <cmath>
using namespace std;

#include "ElementaryFunctions.h"
#include "requests.h"
#include "Entity.h"
#include "Unit.h"
#include "playerUnit.h"
#include "Global.h"
#include "keyStatesClass.h"
#include "mouse.h"

playerUnit::playerUnit() : Unit()
{
	curWeapon = 0;
}

int &playerUnit::getArmor()
{
	return *armorAddress;
}

int &playerUnit::getLife()
{
	return *lifeAddress;
}

void playerUnit::addressInit(int *initAmmo, int *initLife, int *initArmor, int *pistolAddress)
{
	ammoAddress = initAmmo;
	lifeAddress = initLife;
	armorAddress = initArmor;
	pistolAllowed = pistolAddress;
//	for(int i = 0; i < 5; ++i)
//		ammo[i] = initAmmo[i];
}

moveRequest *playerUnit::getMoveRequest(int playerPosX, int playerPosY)
{
	/*if(key[global.keyLeft])
	{
		direction -= 2;
		if(key[global.keyAim])
		{
			//Turn slower
			if(!SPREAD[curWeapon])
				++direction;
		}
		else if(key[global.keyBoost])
		{
			//Turn faster
			direction -= 3;
		}
	}
	if(key[global.keyRight])
	{
		direction += 2;
		if(key[global.keyAim])
		{
			//Turn slower
			if(!SPREAD[curWeapon])
				--direction;
		}
		else if(key[global.keyBoost])
		{
			//Turn faster
			direction += 3;
		}
	}*/
	int destDir = (rad2deg(atan2(int(posX) - (mouse_x + playerPosX), int(posY) - (mouse_y + playerPosY))) * -1) & 0xFF;
	int turnDir;
	if(destDir < 128)
	{
		if(direction < destDir + 128 && direction > destDir)
			turnDir = -1;
		else
			turnDir = 1;
	}
	else
	{
		if(direction < destDir && direction > destDir - 128)
			turnDir = 1;
		else
			turnDir = -1;
	}
	int i;
	for(i = 0; i < 5 && direction != destDir; ++i)
		direction = (direction + turnDir) & 0xFF;

	moveRequest *retVal = new moveRequest;
	if(key[global.keyUp] || key[global.keyDown])
	{
		float newX, newY;
		if(!(key[global.keyLeft] ^ key[global.keyRight]))
		{
			newX = sin(deg2rad(direction)) * speed;
			newY = cos(deg2rad(direction)) * speed * -1.0;
		}
		else
		{
			newX = sin(deg2rad(direction)) * speed * 0.75;
			newY = cos(deg2rad(direction)) * speed * -0.75;
		}
		if(key[global.keyDown])
		{
			newX *= -0.75;
			newY *= -0.75;
		}
		if(key[global.keyRight])
		{
			newX += sin(deg2rad(direction + 64)) * speed * 0.5;
			newY += cos(deg2rad(direction + 64)) * speed * -0.5;
		}
		if(key[global.keyLeft])
		{
			newX += sin(deg2rad(direction - 64)) * speed * 0.5;
			newY += cos(deg2rad(direction - 64)) * speed * -0.5;
		}
		newX += posX;
		newY += posY;
		int scripts = SCRIPT_PLAYER;
		if(keyStates[global.keyUse])
			scripts |= SCRIPT_USE;
		retVal->set(posX, posY, newX, newY, scripts, RIGHT_LOW, radius, 0);

		curState = STATE_WALK;
		++phase &= 15;
	}
	else if(key[global.keyLeft] || key[global.keyRight])
	{
		float newX = 0, newY = 0;
		if(key[global.keyRight])
		{
			newX = sin(deg2rad(direction + 64)) * speed * 0.5;
			newY = cos(deg2rad(direction + 64)) * speed * -0.5;
		}
		if(key[global.keyLeft])
		{
			newX += sin(deg2rad(direction - 64)) * speed * 0.5;
			newY += cos(deg2rad(direction - 64)) * speed * -0.5;
		}
		newX += posX;
		newY += posY;
		int scripts = SCRIPT_PLAYER;
		if(keyStates[global.keyUse])
			scripts |= SCRIPT_USE;
		retVal->set(posX, posY, newX, newY, scripts, RIGHT_LOW, radius, 0);

		phase = 0;
		curState = STATE_WEAPON;
	}
	else
	{
		int scripts = SCRIPT_PLAYER;
		if(keyStates[global.keyUse])
			scripts |= SCRIPT_USE;
		retVal->set(posX, posY, posX, posY, scripts, RIGHT_LOW, radius, 1);

		phase = 0;
		curState = STATE_STAND;
	}

	return retVal;
}

bulletRequest *playerUnit::getBulletRequest()
{
	bulletRequest *retVal = NULL;
	if(keyStates[global.keyNextweapon] || wheelUpEvent() || !getAmmoleft())
	{
		bool secondRun = false;
		while(true)
		{
			++curWeapon;
			if(curWeapon >= 6)
				curWeapon = 0;
			if(!curWeapon)
			{
				if(secondRun || isPistolAllowed())
					break;
				else
					secondRun = true;
			}
			if(curWeapon && getAmmo())
				break;
		}
	}
	if(keyStates[global.keyPreviousweapon] || wheelDownEvent())
	{
		bool secondRun = false;
		while(true)
		{
			--curWeapon;
			if(curWeapon < 0)
				curWeapon = 5;
			if(!curWeapon)
			{
				if(secondRun || isPistolAllowed())
					break;
				else
					secondRun = true;
			}
			if(curWeapon && getAmmo())
				break;
		}
	}
	if(shootCooldown)
		--shootCooldown;
	if(key[global.keyShoot] || mouse_b & 0x01)
	{
		if(!shootCooldown && ((!curWeapon && isPistolAllowed()) || getAmmo()))
		{
			retVal = new bulletRequest;
			retVal->set(posX, posY, direction, curWeapon, this);
			shootCooldown += RATE[curWeapon];
			if(curWeapon)
			{
				--getAmmo();
				if(!getAmmo())
				{
					while(true)
					{
						++curWeapon;
						if(curWeapon >= 6)
							curWeapon = 0;
						if(!curWeapon)
							break;
						if(curWeapon && getAmmo())
							break;
					}
				}
			}
			if(curState == STATE_WALK)
				curState = STATE_WALKSHOOT;
			else if(curState == STATE_STAND || curState == STATE_WEAPON)
				curState = STATE_SHOOT;
		}
		else
		{
			if(curState == STATE_STAND)
				curState = STATE_WEAPON;
		}
	}
	return retVal;
}

int playerUnit::getCurWeapon()
{
	return curWeapon;
}

int playerUnit::getAmmoleft()
{
	if(!curWeapon)
	{
		if(isPistolAllowed())
			return 999;
		else
			return 0;
	}
	return getAmmo();
}

int &playerUnit::getAmmo()
{
	return ammoAddress[curWeapon - 1];
}

void playerUnit::draw(int cameraX, int cameraY)
{
	life = getLife();
	Unit::draw(cameraX, cameraY);
	//Draw aiming line
	if((key[global.keyAim] || mouse_b & 0x02) && isAlife())
	{
		if(SPREAD[curWeapon])
		{
			int targetX1, targetY1, targetX2, targetY2;
			targetX1 = int(posX + sin(deg2rad(direction - SPREAD[curWeapon])) * float(RANGE[curWeapon]) * SPEED[curWeapon]);
			targetY1 = int(posY + cos(deg2rad(direction - SPREAD[curWeapon])) * -1.0 * float(RANGE[curWeapon]) * SPEED[curWeapon]);
			targetX2 = int(posX + sin(deg2rad(direction + SPREAD[curWeapon])) * float(RANGE[curWeapon]) * SPEED[curWeapon]);
			targetY2 = int(posY + cos(deg2rad(direction + SPREAD[curWeapon])) * -1.0 * float(RANGE[curWeapon]) * SPEED[curWeapon]);
			line(global.doublebuffer, int(posX) - cameraX, int(posY) - cameraY, targetX1 - cameraX, targetY1 - cameraY, makecol(0xFF, 0x00, 0x00));
			line(global.doublebuffer, int(posX) - cameraX, int(posY) - cameraY, targetX2 - cameraX, targetY2 - cameraY, makecol(0xFF, 0x00, 0x00));
		}
		else
		{
			int targetX, targetY;
			targetX = int(posX + sin(deg2rad(direction)) * float(RANGE[curWeapon]) * SPEED[curWeapon]);
			targetY = int(posY + cos(deg2rad(direction)) * -1.0 * float(RANGE[curWeapon]) * SPEED[curWeapon]);
			line(global.doublebuffer, int(posX) - cameraX, int(posY) - cameraY, targetX - cameraX, targetY - cameraY, makecol(0xFF, 0x00, 0x00));
		}
	}
}

void playerUnit::load(float speed, int maxLife, int maxArmor, int armType, string spriteData)
{
	Unit::load(speed, maxLife, maxArmor, armType, spriteData, 0, 0);
	getLife() = life;
	getArmor() = armor;
}

bool playerUnit::isAlife()
{
	life = getLife();
	return Unit::isAlife();
}

int playerUnit::getLifePercentage()
{
	life = getLife();
	return Unit::getLifePercentage();
}

int playerUnit::getArmorPercentage()
{
	armor = getArmor();
	return Unit::getArmorPercentage();
}

void playerUnit::getHit(int bulletType)
{
	armor = getArmor();
	life = getLife();
	Unit::getHit(bulletType);
	getArmor() = armor;
	getLife() = life;
}

bool playerUnit::isPistolAllowed()
{
	return !(*pistolAllowed);
}
